package cz.tendersystems.helpdesk.data.general;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.general.*;

public class ApplicationLogData implements Serializable {

	private static final long serialVersionUID = 6217066095134511057L;

	private Long id;

	private String idRequest;

	private LevelRecord level;

	@NotNull
	@Size(min = 2, max = 255)
	private String operation;

	@Size(max = 50)
	private String loginName;

	private Date createdDateTime;

	private String details;

	public ApplicationLogData() {
	}

	public ApplicationLogData(String operation, String details, Object... parameters) {
		this(LevelRecord.INFORMATION, operation, null, MessageFormat.format(details, parameters));
	}

	public ApplicationLogData(LevelRecord level, String operation, String idRequest, String details) {
		this.idRequest = idRequest;
		this.level = level;
		this.operation = operation;
		this.details = details;
	}

	public ApplicationLogData(ApplicationLog entity) {
		this.id = entity.getId();
		this.idRequest = entity.getIdRequest();
		this.level = entity.getLevel();
		this.operation = entity.getOperation();
		this.loginName = entity.getLoginName();
		this.createdDateTime = entity.getCreateDateTime();
		this.details = entity.getDetails();
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getIdRequest() {
		return idRequest;
	}

	public void setIdRequest(String idRequest) {
		this.idRequest = idRequest;
	}

	public LevelRecord getLevel() {
		return level;
	}

	public void setLevel(LevelRecord level) {
		this.level = level;
	}

	public String getOperation() {
		return operation;
	}

	public void setOperation(String operation) {
		this.operation = operation;
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		this.details = details;
	}
}
