package cz.tendersystems.helpdesk.data.organizationalstructure;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.persistence.*;
import javax.validation.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;

public abstract class SystemUserData implements Serializable {

	private static final long serialVersionUID = 1L;

	@Size(min = 5, max = 50)
	@Pattern(regexp = "[a-zA-Z0-9](\\w|[.]|[-]|[@]){4,49}")
	private String LoginName;

	@NotNull
	private Date createdDateTime = new Date();

	@NotNull
	private RoleType roleType;

	@NotNull
	private UserAccountStatus userAccountStatus = UserAccountStatus.ENABLED;

	private ChangePasswordData changePassword;

	@NotNull
	@Embedded
	@Valid
	private PersonData person = new PersonData();

	public SystemUserData() {
	}

	public SystemUserData(SystemUser entity) {
		this.LoginName = entity.getLoginName();
		this.createdDateTime = entity.getCreatedDateTime();
		this.roleType = entity.getRoleType();
		this.userAccountStatus = entity.getUserAccountStatus();
		this.changePassword = entity.getChangePassword() == null ? null : new ChangePasswordData(entity.getChangePassword());
		this.person = new PersonData(entity.getPerson());
	}

	public String getLoginName() {
		return LoginName;
	}

	public void setLoginName(String loginName) {
		LoginName = loginName;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public RoleType getRoleType() {
		return roleType;
	}

	public void setRoleType(RoleType roleType) {
		this.roleType = roleType;
	}

	public UserAccountStatus getUserAccountStatus() {
		return userAccountStatus;
	}

	public void setUserAccountStatus(UserAccountStatus userAccountStatus) {
		this.userAccountStatus = userAccountStatus;
	}

	public ChangePasswordData getChangePassword() {
		return changePassword;
	}

	public void setChangePassword(ChangePasswordData changePassword) {
		this.changePassword = changePassword;
	}

	public PersonData getPerson() {
		return person;
	}

	public void setPerson(PersonData person) {
		this.person = person;
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof SystemUserData && ((SystemUserData) obj).getLoginName().equals(getLoginName());
	}

	@Override
	public int hashCode() {
		return getLoginName() == null ? -1 : getLoginName().hashCode();
	}

	@Override
	public String toString() {
		return MessageFormat.format("{0} (LoginName = {1})", getClass().getSimpleName(), getLoginName());
	}
}
