package cz.tendersystems.helpdesk.data.organizationalstructure.embeddable;

import java.io.*;
import java.text.*;
import javax.validation.constraints.*;
import org.hibernate.validator.constraints.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;

public class PersonData implements Serializable {

	private static final long serialVersionUID = -8749418736391463319L;

	@NotNull
	@Size(min = 1, max = 50)
	private String name;

	@NotNull
	@Size(min = 1, max = 50)
	private String surname;

	@NotNull
	@Email
	@Size(max = 50)
	private String mail;

	@Pattern(regexp = "^[+]\\d{3}\\d{9}$", message = "musí být ve formátu +420xxxxxxxxx")
	@Size(max = 13)
	private String telephone;

	public PersonData() {
	}

	public PersonData(Person object) {
		this.name = object.getName();
		this.surname = object.getSurname();
		this.mail = object.getMail();
		this.telephone = object.getTelephone();
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getMail() {
		return mail;
	}

	public void setMail(String mail) {
		this.mail = mail;
	}

	public String getTelephone() {
		return telephone;
	}

	public void setTelephone(String telephone) {
		this.telephone = telephone;
	}

	@Override
	public String toString() {
		return MessageFormat.format("Name: {0}, Surname: {1}, Mail: {2}, Telephone: {3}", getName(), getSurname(), getMail(), getTelephone());
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof PersonData) {
			PersonData person = (PersonData) obj;

			boolean equalityOfName = getName() != null && getName().equals(person.getName());
			boolean equalityOfSurname = getSurname() != null && getSurname().equals(person.getSurname());
			boolean equalityOfMail = getMail() != null && getMail().equals(person.getMail());
			boolean equalityOfTelephone = getTelephone() != null && getTelephone().equals(person.getTelephone());

			return equalityOfName && equalityOfSurname && equalityOfMail && equalityOfTelephone;
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		int hash = getName() == null ? 47 : getName().hashCode();

		hash += 47 * (getSurname() == null ? 1 : getSurname().hashCode());
		hash += 47 * (getMail() == null ? 1 : getMail().hashCode());
		hash += 47 * (getTelephone() == null ? 1 : getTelephone().hashCode());

		return hash;
	}
}
