package cz.tendersystems.helpdesk.data.request;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.request.*;

public class RequestData implements Serializable {

	private static final long serialVersionUID = 2890485861342731284L;

	@Size(max = 50)
	private String id;

	@NotNull
	@Size(min = 1, max = 255)
	private String name;

	@NotNull
	private RequestStatus requestStatus = RequestStatus.NEW;

	@NotNull
	private Date createdDateTime;

	@NotNull
	private Date reactionTime;

	@NotNull
	private Date timeToSolve;

	private Date dateOfClose;

	@NotNull
	private SlaSchemaData slaSchema;

	@NotNull
	private AuthorData author = new AuthorData();

	private UserSolverData solver;

	public RequestData() {
	}

	public RequestData(Request entity) {
		this.id = entity.getId();
		this.name = entity.getName();
		this.requestStatus = entity.getRequestStatus();
		this.createdDateTime = entity.getCreatedDateTime();
		this.reactionTime = entity.getReactionTime();
		this.timeToSolve = entity.getTimeToSolve();
		this.dateOfClose = entity.getDateOfClose();
		this.slaSchema = new SlaSchemaData(entity.getSlaSchema());
		this.author = new AuthorData(entity.getAuthor());
		this.solver = entity.getSolver() == null ? null : new UserSolverData(entity.getSolver());
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public RequestStatus getRequestStatus() {
		return requestStatus;
	}

	public void setRequestStatus(RequestStatus requestStatus) {
		this.requestStatus = requestStatus;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public Date getReactionTime() {
		return reactionTime;
	}

	public void setReactionTime(Date reactionTime) {
		this.reactionTime = reactionTime;
	}

	public Date getTimeToSolve() {
		return timeToSolve;
	}

	public void setTimeToSolve(Date timeToSolve) {
		this.timeToSolve = timeToSolve;
	}

	public Date getDateOfClose() {
		return dateOfClose;
	}

	public void setDateOfClose(Date dateOfClose) {
		this.dateOfClose = dateOfClose;
	}

	public SlaSchemaData getSlaSchema() {
		return slaSchema;
	}

	public void setSlaSchema(SlaSchemaData slaSchema) {
		this.slaSchema = slaSchema;
	}

	public AuthorData getAuthor() {
		return author;
	}

	public void setAuthor(AuthorData author) {
		this.author = author;
	}

	public UserSolverData getSolver() {
		return solver;
	}

	public void setSolver(UserSolverData solver) {
		this.solver = solver;
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof RequestData && ((RequestData) obj).getId().equals(getId());
	}

	@Override
	public int hashCode() {
		return getId() == null ? -1 : getId().hashCode();
	}

	@Override
	public String toString() {
		return MessageFormat.format("{0} (id = {1})", getClass().getSimpleName(), getId());
	}
}
