package cz.tendersystems.helpdesk.entity.general;

import java.util.*;
import javax.persistence.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.entity.enums.*;

@Entity
public class ApplicationLog extends AbstractEntity<Long> {

	private static final long serialVersionUID = 2972985007754310674L;

	@Id
	@GeneratedValue(strategy = GenerationType.TABLE, generator = "application_log")
	@TableGenerator(name = "application_log", allocationSize = 1, initialValue = 0)
	private Long id;

	private String idRequest;

	@Enumerated(EnumType.STRING)
	private LevelRecord level;

	@NotNull
	@Size(min = 2, max = 255)
	private String operation;

	@Size(max = 50)
	private String loginName;

	@Temporal(TemporalType.TIMESTAMP)
	private Date createdDateTime;

	@Column(columnDefinition = "TEXT")
	private String details;

	public ApplicationLog() {
		createdDateTime = new Date();
	}

	public ApplicationLog(ApplicationLogData data) {
		this();
		convert(this, data);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getIdRequest() {
		return idRequest;
	}

	public void setIdRequest(String idRequest) {
		this.idRequest = idRequest;
	}

	public LevelRecord getLevel() {
		return level;
	}

	public void setLevel(LevelRecord level) {
		this.level = level;
	}

	public String getOperation() {
		return operation;
	}

	public void setOperation(String operation) {
		this.operation = operation;
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public Date getCreateDateTime() {
		return createdDateTime;
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		this.details = details;
	}

	public static void convert(ApplicationLog entity, ApplicationLogData data) {
		entity.setIdRequest(data.getIdRequest());
		entity.setLevel(data.getLevel());
		entity.setOperation(data.getOperation());
		entity.setLoginName(data.getLoginName());
		entity.setDetails(data.getDetails());
	}
}
