package cz.tendersystems.helpdesk.entity.organizationalstructure;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.persistence.*;
import javax.validation.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;

@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
public abstract class SystemUser implements Serializable {

	private static final long serialVersionUID = 3957253466916517266L;

	@Id
	@Size(min = 5, max = 50)
	@Pattern(regexp = "[a-zA-Z0-9](\\w|[.]|[-]|[@]){4,49}")
	private String loginName;

	@NotNull
	@Pattern(regexp = "[a-fA-F0-9]{40}")
	private String password;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date createdDateTime;

	@NotNull
	@Enumerated(EnumType.STRING)
	private RoleType roleType;

	@NotNull
	@Enumerated(EnumType.STRING)
	private UserAccountStatus userAccountStatus;

	@Valid
	@Embedded
	private ChangePassword changePassword;

	@NotNull
	@Embedded
	@Valid
	private Person person;

	public SystemUser() {
	}

	public SystemUser(SystemUserData data) {
		convert(this, data);
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public RoleType getRoleType() {
		return roleType;
	}

	public void setRoleType(RoleType roleType) {
		this.roleType = roleType;
	}

	public UserAccountStatus getUserAccountStatus() {
		return userAccountStatus;
	}

	public void setUserAccountStatus(UserAccountStatus userAccountStatus) {
		this.userAccountStatus = userAccountStatus;
	}

	public Person getPerson() {
		return person;
	}

	public void setPerson(Person person) {
		this.person = person;
	}

	public ChangePassword getChangePassword() {
		return changePassword;
	}

	public void setChangePassword(ChangePassword changePassword) {
		this.changePassword = changePassword;
	}

	public static void convert(SystemUser entity, SystemUserData data) {
		entity.setCreatedDateTime(data.getCreatedDateTime());
		entity.setRoleType(data.getRoleType());
		entity.setUserAccountStatus(data.getUserAccountStatus());
		entity.setPerson(new Person(data.getPerson()));
		entity.setChangePassword(data.getChangePassword() == null ? null : new ChangePassword(data.getChangePassword()));
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof SystemUser && ((SystemUser) obj).getLoginName().equals(getLoginName());
	}

	@Override
	public int hashCode() {
		return getLoginName() == null ? -1 : getLoginName().hashCode();
	}

	@Override
	public String toString() {
		return MessageFormat.format("{0} (login name = {1})", getClass().getSimpleName(), getLoginName());
	}
}
