package cz.tendersystems.helpdesk.entity.organizationalstructure.sla;

import java.util.*;
import javax.persistence.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.general.*;

@Entity
public class SlaSchema extends AbstractEntity<Long> {

	private static final long serialVersionUID = -7074649189892416038L;

	@Id
	@GeneratedValue(strategy = GenerationType.TABLE, generator = "sla_schema")
	@TableGenerator(name = "sla_schema", allocationSize = 1, initialValue = 0)
	private Long id;

	@NotNull
	@Size(min = 1, max = 255)
	private String name;

	@NotNull
	@Temporal(TemporalType.TIME)
	private Date reactionTime;

	@NotNull
	@Temporal(TemporalType.TIME)
	private Date timeToSolve;

	@NotNull
	private Boolean active = true;

	@NotNull
	@Size(min = 1, max = 8)
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "slaSchema", cascade = { CascadeType.PERSIST, CascadeType.REMOVE })
	private List<TimeActivity> timesActivity = new ArrayList<>();

	public SlaSchema() {
	}

	public SlaSchema(SlaSchemaData data) {
		id = data.getId();
		convert(this, data);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Date getReactionTime() {
		return reactionTime;
	}

	public void setReactionTime(Date reactionTime) {
		this.reactionTime = reactionTime;
	}

	public Date getTimeToSolve() {
		return timeToSolve;
	}

	public void setTimeToSolve(Date timeToSolve) {
		this.timeToSolve = timeToSolve;
	}

	public Boolean getActive() {
		return active;
	}

	public void setActive(Boolean active) {
		this.active = active;
	}

	public List<TimeActivity> getTimesActivity() {
		return timesActivity;
	}

	public void setTimesActivity(List<TimeActivity> timesActivity) {
		this.timesActivity = timesActivity;
	}

	public static void convert(SlaSchema entity, SlaSchemaData data) {
		entity.setName(data.getName());
		entity.setReactionTime(data.getReactionTime());
		entity.setTimeToSolve(data.getTimeToSolve());
		entity.setActive(data.getActive());
	}
}
