package cz.tendersystems.helpdesk.entity.request;

import java.io.*;
import javax.persistence.*;
import javax.validation.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.general.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;

@Entity
public class Author extends AbstractEntity<Long> implements Serializable {

	private static final long serialVersionUID = 4549394248413737332L;

	@Id
	@GeneratedValue(strategy = GenerationType.TABLE, generator = "author")
	@TableGenerator(name = "author", allocationSize = 1, initialValue = 0)
	private Long id;

	@Size(min = 1, max = 50)
	private String companyName;

	@Embedded
	@Valid
	private Person anonymous;

	@ManyToOne(fetch = FetchType.LAZY)
	private UserOutsourcer loggedUser;

	public Author() {
	}

	public Author(AuthorData data) {
		convert(this, data);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getCompanyName() {
		return companyName;
	}

	public void setCompanyName(String companyName) {
		this.companyName = companyName;
	}

	public Person getAnonymous() {
		return anonymous;
	}

	public void setAnonymous(Person anonymous) {
		this.anonymous = anonymous;
	}

	public UserOutsourcer getLoggedUser() {
		return loggedUser;
	}

	public void setLoggedUser(UserOutsourcer loggedUser) {
		this.loggedUser = loggedUser;
	}

	public static void convert(Author entity, AuthorData data) {
		entity.setCompanyName(data.getCompanyName());
		entity.setAnonymous(data.getAnonymous() == null ? null : new Person(data.getAnonymous()));
		entity.setLoggedUser(data.getLoggedUser() == null ? null : new UserOutsourcer(data.getLoggedUser()));
	}
}
