package cz.tendersystems.helpdesk.entity.request;

import java.util.*;
import javax.persistence.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.general.*;

@Entity
public class Comment extends AbstractEntity<Long> {

	private static final long serialVersionUID = 8961104053095802762L;

	@Id
	@GeneratedValue(strategy = GenerationType.TABLE, generator = "comment")
	@TableGenerator(name = "comment", allocationSize = 1, initialValue = 0)
	private Long id;

	@NotNull
	@Column(columnDefinition = "TEXT")
	private String text;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date createdDateTime;

	@NotNull
	@Enumerated(EnumType.STRING)
	private RequestStatus requestStatus = RequestStatus.NEW;

	@Size(max = 5)
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "comment", cascade = CascadeType.PERSIST)
	private List<File> attachments;

	@NotNull
	@Size(min = 1, max = 255)
	private String author;

	@NotNull
	@ManyToOne(fetch = FetchType.LAZY)
	private Request request;

	public Comment() {
	}

	public Comment(CommentData data) {
		convert(this, data);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public RequestStatus getRequestStatus() {
		return requestStatus;
	}

	public void setRequestStatus(RequestStatus requestStatus) {
		this.requestStatus = requestStatus;
	}

	public List<File> getAttachments() {
		return attachments;
	}

	public void setAttachments(List<File> attachments) {
		this.attachments = attachments;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public Request getRequest() {
		return request;
	}

	public void setRequest(Request request) {
		this.request = request;
	}

	public static void convert(Comment entita, CommentData data) {
		entita.setText(data.getText());
		entita.setCreatedDateTime(data.getCreatedDateTime());
		entita.setRequestStatus(data.getRequestStatus());
		entita.setAuthor(data.getAuthor());
		;
	}
}
