package cz.tendersystems.helpdesk.entity.request;

import java.util.*;
import javax.persistence.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.general.*;

@Entity
public class File extends AbstractEntity<Long> {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.TABLE, generator = "file")
	@TableGenerator(name = "file", allocationSize = 1, initialValue = 0)
	private Long id;

	@NotNull
	@Size(min = 1, max = 255)
	private String name;

	@NotNull
	@Min(1)
	private Long size;

	@NotNull
	@Size(min = 1, max = 150)
	private String contentType;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date createdDateTime;

	@NotNull
	@ManyToOne(fetch = FetchType.LAZY)
	private Comment comment;

	public File() {
	}

	public File(FileData data) {
		convert(this, data);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getContentType() {
		return contentType;
	}

	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	public Long getSize() {
		return size;
	}

	public void setSize(Long size) {
		this.size = size;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public Comment getComment() {
		return comment;
	}

	public void setComment(Comment comment) {
		this.comment = comment;
	}

	public static void convert(File entity, FileData data) {
		entity.setName(data.getName());
		entity.setContentType(data.getContentType());
		entity.setSize(data.getSize());
		entity.setCreatedDateTime(data.getCreatedDateTime());
	}
}
