package cz.tendersystems.helpdesk.entity.request;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.persistence.*;
import javax.validation.constraints.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;

@Entity
public class Request implements Serializable {

	private static final long serialVersionUID = 5284725544770998550L;

	@Id
	@Size(max = 50)
	private String id;

	@NotNull
	@Size(min = 1, max = 255)
	private String name;

	@NotNull
	@Enumerated(EnumType.STRING)
	private RequestStatus requestStatus = RequestStatus.NEW;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date createdDateTime;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date reactionTime;

	@NotNull
	@Temporal(TemporalType.TIMESTAMP)
	private Date timeToSolve;

	@Temporal(TemporalType.TIMESTAMP)
	private Date dateOfClose;

	@NotNull
	@ManyToOne(fetch = FetchType.LAZY)
	private SlaSchema slaSchema;

	@NotNull
	@ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.PERSIST)
	private Author author;

	@ManyToOne(fetch = FetchType.LAZY)
	private UserSolver solver;

	@NotNull
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "request")
	private List<Comment> comments;

	public Request() {
	}

	public Request(RequestData data) {
		convert(this, data);
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public RequestStatus getRequestStatus() {
		return requestStatus;
	}

	public void setRequestStatus(RequestStatus requestStatus) {
		this.requestStatus = requestStatus;
	}

	public Date getCreatedDateTime() {
		return createdDateTime;
	}

	public void setCreatedDateTime(Date createdDateTime) {
		this.createdDateTime = createdDateTime;
	}

	public Date getReactionTime() {
		return reactionTime;
	}

	public void setReactionTime(Date reactionTime) {
		this.reactionTime = reactionTime;
	}

	public Date getTimeToSolve() {
		return timeToSolve;
	}

	public void setTimeToSolve(Date timeToSolve) {
		this.timeToSolve = timeToSolve;
	}

	public SlaSchema getSlaSchema() {
		return slaSchema;
	}

	public void setSlaSchema(SlaSchema slaSchema) {
		this.slaSchema = slaSchema;
	}

	public Author getAuthor() {
		return author;
	}

	public void setAuthor(Author author) {
		this.author = author;
	}

	public UserSolver getSolver() {
		return solver;
	}

	public void setSolver(UserSolver solver) {
		this.solver = solver;
	}

	public List<Comment> getComments() {
		return comments;
	}

	public void setComments(List<Comment> comments) {
		this.comments = comments;
	}

	public Date getDateOfClose() {
		return dateOfClose;
	}

	public void setDateOfClose(Date dateOfClose) {
		this.dateOfClose = dateOfClose;
	}

	public static void convert(Request entita, RequestData data) {
		entita.setId(data.getId());
		entita.setName(data.getName());
		entita.setRequestStatus(data.getRequestStatus());
		entita.setCreatedDateTime(data.getCreatedDateTime());
		entita.setReactionTime(data.getReactionTime());
		entita.setTimeToSolve(data.getTimeToSolve());
		entita.setDateOfClose(data.getDateOfClose());
		entita.setSlaSchema(new SlaSchema(data.getSlaSchema()));
		entita.setAuthor(new Author(data.getAuthor()));
		entita.setSolver(data.getSolver() == null ? null : new UserSolver(data.getSolver()));
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof Request && ((Request) obj).getId().equals(getId());
	}

	@Override
	public int hashCode() {
		return getId() == null ? -1 : getId().hashCode();
	}

	@Override
	public String toString() {
		return MessageFormat.format("{0} (id = {1})", getClass().getSimpleName(), getId());
	}
}
