package cz.tendersystems.helpdesk.general;

import java.io.*;
import java.math.*;
import java.security.*;
import java.text.*;
import java.util.*;
import org.hibernate.proxy.*;

public class Tools {

	public static final String FILE_PATH_SEPARATOR = System.getProperty("file.separator");
	public static final String SERVER_DATA_FOLDER_PATH = System.getProperty("jboss.server.data.dir");

	private static final ResourceBundle text = ResourceBundle.getBundle("cz.tendersystems.helpdesk.resources.messages");
	private static final int stackSize = 4096;
	public static final String CHARSET = "UTF-8";

	public static final String LOCALIZATION = "cs_CZ";
	public static final Locale LOCALIZATION_OF_FORMATTER = new Locale("cs", "CZ");

	public static final NumberFormat FORMATTER_OF_WHOLE_NUMBERS = new DecimalFormat("###,###,##0", new DecimalFormatSymbols(LOCALIZATION_OF_FORMATTER));
	public static final NumberFormat FORMATTER_OF_FILE_SIZE = new DecimalFormat("###,###,##0.#", new DecimalFormatSymbols(LOCALIZATION_OF_FORMATTER));

	private static final Random generator = new Random();
	private static final String allowedCharacters = "0123456789abcdefghaijklmnopqrstuvwxyz";
	private static final String hashFunction = "SHA1";

	@SuppressWarnings("unchecked")
	public static <E> E prepareEntityOfRightType(E entity) {
		return entity instanceof HibernateProxy ? (E) ((HibernateProxy) entity).getHibernateLazyInitializer().getImplementation() : entity;
	}

	public static String generateRandomString(Integer lenght) {
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < lenght; i++) {
			sb.append(allowedCharacters.charAt(generator.nextInt(allowedCharacters.length())));
		}

		return sb.toString();
	}

	public static String generateHash(String stringValue, String sul) {
		try {
			return generatedHash(new ByteArrayInputStream(stringValue.getBytes(CHARSET)), sul);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}

	private static String generatedHash(InputStream input, String sul) {
		try {
			MessageDigest md = MessageDigest.getInstance(hashFunction);
			byte[] dataStack = new byte[stackSize];
			int readCount;

			while ((readCount = input.read(dataStack)) != -1) {
				md.update(dataStack, 0, readCount);
			}

			md.update(sul.getBytes(CHARSET));

			return String.format("%040x", new BigInteger(1, md.digest()));
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static String generateLink(String aplicationDomain, String page, Map<String, String> parameters) {
		StringBuffer link = new StringBuffer();
		link.append(aplicationDomain).append(page);

		int parametrCount = 0;
		for (String key : parameters.keySet()) {
			link.append(parametrCount++ == 0 ? "?" : "&");
			link.append(key).append("=").append(parameters.get(key));
		}

		return link.toString();
	}

	public static String getEnumAsText(Enum<?> enumObject) {
		String keyStringValue = getKeyStringValue(enumObject.getClass().getSimpleName(), enumObject.name());

		return existKeyStringValue(keyStringValue) ? text.getString(keyStringValue) : keyStringValue;
	}

	public static String getText(String keyStringValue) {
		return existKeyStringValue(keyStringValue) ? text.getString(keyStringValue) : keyStringValue;
	}

	private static boolean existKeyStringValue(String keyStringValue) {
		return text.containsKey(keyStringValue);
	}

	public static String getKeyStringValue(String enumType, String enumConstant) {
		return MessageFormat.format("{0}.{1}", enumType.toLowerCase(), enumConstant.toLowerCase());
	}

	public static String provideDateAsText(Date date, DateAndTimeFormat formatType) {
		return formatType.provideDateAsText(date);
	}

	public static String provideDateWithTime(Date date) {
		return DateAndTimeFormat.FORMAT_OF_DATE_AND_TIME.provideDateAsText(date);
	}

	public static String getPathToServerFolder(String folderName) {
		return MessageFormat.format("{0}{1}{2}", SERVER_DATA_FOLDER_PATH, FILE_PATH_SEPARATOR, folderName);
	}

	public static String getFilePathSeparator() {
		return FILE_PATH_SEPARATOR;
	}

	public static boolean isEnum(Class<?> type) {
		return type != null && Enum.class.isAssignableFrom(type);
	}

	public static boolean isNumber(Class<?> type) {
		return type != null && Number.class.isAssignableFrom(type);
	}
}
