package cz.tendersystems.helpdesk.general;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.http.*;
import org.apache.commons.io.*;
import org.jboss.logging.*;

public class ToolsForBinaryDownload {

	private enum LogSeverity {
		INFO, WARN, ERROR;
	}

	private static final Logger LOG = Logger.getLogger(ToolsForBinaryDownload.class);

	private static final String headerKey = "Content-Disposition";
	private static final String heaaderValue = "attachment; filename=\"%s\";";
	private static final String defaultName = "unknown_file";

	public static void downloadFile(HttpServletRequest request, HttpServletResponse response, InputStream binaryFileIs, String fileName, int fileSize) {
		log(LogSeverity.INFO, MessageFormat.format("Soubor {0} o velikosti {1} B", fileName, fileSize));

		try {
			setResponse(request, response, binaryFileIs, fileName, fileSize);
		} catch (Exception e) {
			log(LogSeverity.ERROR, "Neocekavana chyba pri stazeni souboru", e);
			response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		} finally {
			try {
				response.getOutputStream().close();
			} catch (Exception e) {
				log(LogSeverity.ERROR, "Neocekavana chyba pri stazeni souboru", e);
			}
		}
	}

	private static void setResponse(HttpServletRequest request, HttpServletResponse response, InputStream binaryFileIs, String fileName, int fileSize) throws IOException {
		response.setHeader(headerKey, String.format(heaaderValue, setFileNameEncoding(fileName == null ? defaultName : fileName, request, response.getCharacterEncoding())));
		response.setContentType(File.pathSeparator);
		response.setContentLength(fileSize);
		response.setStatus(HttpServletResponse.SC_OK);

		IOUtils.copy(binaryFileIs, response.getOutputStream());
	}

	private static String setFileNameEncoding(String fileName, HttpServletRequest request, String responseEncoding) throws UnsupportedEncodingException {
		Enumeration<String> headerNames = request.getHeaderNames();

		while (headerNames.hasMoreElements()) {
			String headerName = headerNames.nextElement();
			System.out.println(headerName + " : " + request.getHeader(headerName));
		}

		return new String(removeDiacritics(fileName).getBytes("Windows-1250"), responseEncoding);
	}

	private static String removeDiacritics(String stringValue) {
		return (Normalizer.normalize(stringValue, Normalizer.Form.NFD)).replaceAll("\\p{InCombiningDiacriticalMarks}+", "");
	}

	private static void log(LogSeverity severity, String text, Object... parameters) {
		if (severity.equals(LogSeverity.WARN)) {
			LOG.warnv(text, parameters);
		} else if (severity.equals(LogSeverity.ERROR)) {
			LOG.errorv(text, parameters);
		} else if (severity.equals(LogSeverity.INFO)) {
			LOG.infov(text, parameters);
		} else {
			System.out.println(MessageFormat.format(text, parameters));
		}
	}
}