package cz.tendersystems.helpdesk.general;

import java.text.*;

public class ToolsForFileSize {

	private static final MultipleOfFileSize defaultMultiple = MultipleOfFileSize.DECIMAL;
	private static final String format = "{0} {1}";

	public static String convertToString(Long size) {
		return convertToString(size, defaultMultiple);
	}

	private static String convertToString(Long size, MultipleOfFileSize multiple) {
		int index = getIndex(size, multiple);

		return MessageFormat.format(format, getValue(size, multiple, index), multiple.getUnits()[index]);
	}

	private static int getIndex(Long size, MultipleOfFileSize multiple) {
		return Math.min(size > 0 ? (int) (Math.log10(size) / Math.log10(multiple.getValue())) : 0, multiple.getUnits().length - 1);
	}

	private static String getValue(Long size, MultipleOfFileSize multiple, int index) {
		return Tools.FORMATTER_OF_FILE_SIZE.format(index == 0 ? size : ((double) size) / Math.pow(multiple.getValue(), index));
	}

	public enum MultipleOfFileSize {

		DECIMAL(1000, new String[] { "B", "kB", "MB", "GB", "TB" }),
		BINARY_IEC(1024, new String[] { "B", "KiB", "MiB", "GiB", "TiB" });

		private double value;
		private String[] units;

		private MultipleOfFileSize(double value, String[] units) {
			this.value = value;
			this.units = units;
		}

		public double getValue() {
			return value;
		}

		public String[] getUnits() {
			return units;
		}
	}
}
