package cz.tendersystems.helpdesk.general;

import java.time.*;
import java.util.*;

public class ToolsForHolidays {

	private static Map<Integer, Integer[]> holidayMap = new HashMap<>();
	private static Map<Integer, Integer[]> easterMap = new HashMap<>();

	public static boolean isHolidayDay(LocalDate date) {
		prepareHolidays(date.getYear());

		return isDayInHolidayMap(date) || isDayInEasterMap(date);
	}

	private static boolean isDayInHolidayMap(LocalDate date) {
		Integer[] listHolidayDay = holidayMap.get(date.getMonthValue());

		if (listHolidayDay == null) {
			return false;
		}

		for (Integer i : listHolidayDay) {
			if (date.getDayOfMonth() == i) {
				return true;
			}
		}

		return false;
	}

	private static boolean isDayInEasterMap(LocalDate date) {
		Integer[] listEasterDay = easterMap.get(date.getMonthValue());

		if (listEasterDay == null) {
			return false;
		}

		for (Integer i : listEasterDay) {
			if (date.getDayOfMonth() == i) {
				return true;
			}
		}

		return false;
	}

	private static void prepareHolidays(int year) {
		if (holidayMap.isEmpty() || easterMap.isEmpty()) {
			prepareHolidaysMap();
			prepareEaster(year);
		}
	}

	// TODO klíč = měsíc, hodnota= seznam dnů
	private static void prepareHolidaysMap() {
		holidayMap = new HashMap<>();

		holidayMap.put(1, new Integer[] { 1 });
		holidayMap.put(5, new Integer[] { 1, 8 });
		holidayMap.put(7, new Integer[] { 5, 6 });
		holidayMap.put(9, new Integer[] { 28 });
		holidayMap.put(10, new Integer[] { 28 });
		holidayMap.put(11, new Integer[] { 17 });
		holidayMap.put(12, new Integer[] { 24, 25, 26 });
	}

	private static void prepareEaster(int year) {
		LocalDate actualDate = LocalDate.now();

		int goldNumber = (year % 19) + 1;
		int century = (year / 100) + 1;
		int solarRepair = (3 * century / 4) - 12;
		int moonRepair = ((8 * century + 5) / 25) - 5;
		int correction = (5 * year / 4) - solarRepair - 10;
		int epact = (11 * goldNumber + 20 + moonRepair - solarRepair) % 30;

		if ((epact == 25 && goldNumber > 11) || epact == 24) {
			epact++;
		}

		int cyclicFullMoon = 44 - epact;

		cyclicFullMoon += 30 * (cyclicFullMoon < 21 ? 1 : 0);

		cyclicFullMoon += 7 - ((correction + cyclicFullMoon) % 7);

		if (cyclicFullMoon > 31) {
			actualDate = LocalDate.of(year, 4, cyclicFullMoon - 31);
		} else {
			actualDate = LocalDate.of(year, 3, cyclicFullMoon);
		}

		actualDate = actualDate.plusDays(1);

		LocalDate bigFriday = actualDate.minusDays(3);
		easterMap.put(actualDate.getMonthValue(), new Integer[] { actualDate.getDayOfMonth(), bigFriday.getDayOfMonth() });
	}
}
