package cz.tendersystems.helpdesk.general;

import java.text.*;
import java.time.*;
import java.time.format.*;
import java.util.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;

public class ToolsForRequestDates {

	private static LocalDateTime actualDate;
	private static LocalTime updateTime;

	private static LocalDateTime reactionTime;
	private static LocalDateTime timeToSolve;

	public void calculateRequestDates(Request request) {
		calculateReactionTime(request);
		calculateTimeToSolve(request.getSlaSchema());
	}

	private static void calculateReactionTime(Request request) {
		SlaSchema slaSchema = request.getSlaSchema();

		actualDate = LocalDateTime.ofInstant(request.getCreatedDateTime().toInstant(), ZoneId.systemDefault());
		updateTime = convertDateToLocalTime(slaSchema.getReactionTime());

		reactionTime = prepareDate(slaSchema.getTimesActivity());
	}

	private static void calculateTimeToSolve(SlaSchema slaSchema) {
		actualDate = reactionTime;
		updateTime = convertDateToLocalTime(slaSchema.getTimeToSolve());

		timeToSolve = prepareDate(slaSchema.getTimesActivity());
	}

	private static LocalDateTime prepareDate(List<TimeActivity> timesActivity) {
		TimeActivity timeAcitivityForHoliday = prepareTimeActivtyForHoliday(timesActivity);

		for (TimeActivity timeActivity : timesActivity) {
			if (predicateForWorkDay(timeActivity) || predicateForHoliday(timeAcitivityForHoliday)) {
				if (timeAcitivityForHoliday == null && ToolsForHolidays.isHolidayDay(actualDate.toLocalDate())) {
					break;
				}

				prepareTime(ToolsForHolidays.isHolidayDay(actualDate.toLocalDate()) ? timeAcitivityForHoliday : timeActivity);

				if (updateTime.compareTo(LocalTime.of(0, 0, 0)) == 1) {
					prepareDate(timesActivity);
				}

				return actualDate;
			}
		}

		actualDate = LocalDateTime.of(actualDate.getYear(), actualDate.getMonth(), actualDate.getDayOfMonth(), 0, 0, 0).plusDays(1);

		prepareDate(timesActivity);

		return actualDate;
	}

	private static TimeActivity prepareTimeActivtyForHoliday(List<TimeActivity> timesActivity) {
		for (TimeActivity timeActivity : timesActivity) {
			if (DayName.HOLIDAY.equals(timeActivity.getDayName())) {
				return timeActivity;
			}
		}

		return null;
	}

	private static boolean predicateForWorkDay(TimeActivity timeActivity) {
		return actualDate.getDayOfWeek().toString().equals(timeActivity.getDayName().toString()) && actualDate.toLocalTime().compareTo(convertDateToLocalTime(timeActivity.getEndTime())) == -1;
	}

	private static boolean predicateForHoliday(TimeActivity timeActivity) {
		return timeActivity != null && ToolsForHolidays.isHolidayDay(actualDate.toLocalDate()) && actualDate.toLocalTime().compareTo(convertDateToLocalTime(timeActivity.getEndTime())) == -1;
	}

	private static void prepareTime(TimeActivity timeActivity) {
		if (actualDate.toLocalTime().compareTo(convertDateToLocalTime(timeActivity.getStartTime())) == -1) {
			LocalTime tempTime = convertDateToLocalTime(timeActivity.getStartTime());
			actualDate = LocalDateTime.of(actualDate.getYear(), actualDate.getMonth(), actualDate.getDayOfMonth(), tempTime.getHour(), tempTime.getMinute(), tempTime.getSecond());
		}

		actualDate = actualDate.plusHours(updateTime.getHour()).plusMinutes(updateTime.getMinute()).plusSeconds(updateTime.getSecond());

		if (actualDate.toLocalTime().compareTo(convertDateToLocalTime(timeActivity.getEndTime())) == 1 || actualDate.toLocalTime().compareTo(convertDateToLocalTime(timeActivity.getStartTime())) == -1) {
			updateTime = calculateDivideTime(timeActivity.getEndTime());
		} else {
			updateTime = LocalTime.of(0, 0, 0);
		}
	}

	private static LocalTime calculateDivideTime(Date time) {
		LocalTime tempTime = convertDateToLocalTime(time);

		return actualDate.toLocalTime().minusHours(tempTime.getHour()).minusMinutes(tempTime.getMinute()).minusSeconds(tempTime.getSecond());
	}

	private static LocalTime convertDateToLocalTime(Date date) {
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		String time = sdf.format(date);

		return LocalTime.parse(time, DateTimeFormatter.ofPattern("HH:mm:ss"));
	}

	private static Date convertLocalDateTimeToDate(LocalDateTime date) {
		return Date.from(date.atZone(ZoneId.systemDefault()).toInstant());
	}

	public Date getReactionTime() {
		return convertLocalDateTimeToDate(reactionTime);
	}

	public Date getTimeToSolve() {
		return convertLocalDateTimeToDate(timeToSolve);
	}
}
