package cz.tendersystems.helpdesk.general.validation;

import java.util.*;
import java.util.regex.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationPassword {

	private static final Pattern illeagalPasswordCharacter = Pattern.compile("\\s");
	private static final Integer passwordLenght = 8;
	private static final String passwordRegex = "^(?=.*?[A-Z])(?=(.*[a-z]){1,})(?=(.*[\\d]){1,})(?=(.*[\\W]){1,})(?!.*\\s).{8,}$";

	public static void validatePassword(String newPassword, SystemUser user) throws ValidationException {
		if (!isValidKey(user)) {
			throw new ValidationException("error.keyIsInvalid");
		}

		if (illeagalPasswordCharacter.matcher(newPassword).find()) {
			throw new ValidationException("error.passwordContainsSpaces");
		}

		if (newPassword.length() < passwordLenght) {
			throw new ValidationException("error.passwordContainsAFewCharacters", passwordLenght);
		}

		if (!newPassword.matches(passwordRegex)) {
			throw new ValidationException("error.passwordDoesNotMatchFormat");
		}
	}

	public static boolean isValidKey(SystemUser user) {
		return user != null && user.getChangePassword().getExpirationDate().after(new Date());
	}
}
