package cz.tendersystems.helpdesk.general.validation;

import java.util.*;
import java.util.regex.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationUser {

	private static final Pattern illeagalUserCharacter = Pattern.compile("\\s");
	private static final String userRegex = "[a-zA-Z0-9](\\w|[.]|[-]|[@]){4,49}";

	public static void validateUser(SystemUserData user) throws ValidationException {
		if (user.getLoginName() != null) {
			if (illeagalUserCharacter.matcher(user.getLoginName()).find()) {
				throw new ValidationException("error.loginNameContainsSpaces");
			}

			if (!user.getLoginName().matches(userRegex)) {
				throw new ValidationException("error.loginNameDoesNotMatchFormat");
			}
		}
		validateUserOutsourcer(user);
		validateUserSolver(user);
	}

	private static void validateUserOutsourcer(SystemUserData user) throws ValidationException {
		if (user instanceof UserSolverData && user.getRoleType().equals(RoleType.OUTSOURCER)) {
			throw new ValidationException("error.userSolverCanNotBeRoleTypeOutsourcer");
		}

		if (user instanceof UserOutsourcerData && !user.getRoleType().equals(RoleType.OUTSOURCER)) {
			throw new ValidationException("error.userOutsourcerMustHasRoleTypeOutsourcer");
		}
	}

	private static void validateUserSolver(SystemUserData user) throws ValidationException {
		if (user instanceof UserSolverData) {
			if (user.getRoleType().equals(RoleType.ADMINISTRATOR) && !((UserSolverData) user).getProducts().isEmpty()) {
				throw new ValidationException("error.userRoleAdministratorHasNotProducts");
			}

			if (Arrays.asList(RoleType.SOLVER, RoleType.WATCHER).contains(user.getRoleType()) && ((UserSolverData) user).getProducts().isEmpty()) {
				throw new ValidationException("error.userMustBeProduct");
			}
		} else {
			if (((UserOutsourcerData) user).getOutsourcer() == null) {
				throw new ValidationException("error.userOutsourcerMustBeOutsourcer");
			}

			if (!((UserOutsourcerData) user).getOutsourcer().getActive()) {
				throw new ValidationException("error.selectedOutsourcerMustBeActive");
			}
		}
	}
}
