package cz.tendersystems.helpdesk.service.configuration;

import java.text.*;
import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.naming.*;
import org.jboss.logging.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.*;

@Singleton
@LocalBean
@Startup
public class ConfigurationService {

	private static final String pathToConfiguration = "cz.tendersystems.helpdesk.configuration";
	private static final String pathToVersion = "cz.tendersystems.helpdesk.version";

	private VersionType versionType;
	private String applicationDomain;
	private String revisionNumber;
	private String applicationName;
	private String informationMail;
	private String providerPage;
	private String providerMail;
	private String providerTelephone;
	private String binaryFilesFolder;

	@PostConstruct
	public void loadConfiguration() {
		try {
			Logger.getLogger(getClass()).infov("Nacitam nastaveni");

			ResourceBundle configuration = ResourceBundle.getBundle(pathToConfiguration);
			ResourceBundle version = ResourceBundle.getBundle(pathToVersion);

			versionType = VersionType.valueOf(configuration.getString("versionType"));
			applicationDomain = configuration.getString("applicationDomain");
			revisionNumber = version.getString("revisionNumber");
			applicationName = ((String) new InitialContext().lookup("java:app/AppName"));
			informationMail = configuration.getString("informationMail");
			providerPage = configuration.getString("providerPage");
			providerMail = configuration.getString("provederMail");
			providerTelephone = configuration.getString("providerTelephone");
			binaryFilesFolder = configuration.getString("binaryFilesFolder");

			Logger.getLogger(getClass()).infov("Verze {0} / {1} / {2} / {3}", getVersionName(), versionType, revisionNumber, applicationName);

			Logger.getLogger(getClass()).infov("Nastaveni bylo uspesne nacteno");
		} catch (Exception e) {
			throw new HelpdeskException("Chyba pri nacitani nastaveni", e);
		}
	}

	public String getEjbJndi(String ejbName) {
		return MessageFormat.format("java:global/{0}/{1}-ejb/{2}", applicationName, applicationName.replace("-ear", ""), ejbName);
	}

	public String getVersionName() {
		return Tools.getEnumAsText(versionType);
	}

	public String getApplicationDomain() {
		return applicationDomain;
	}

	public String getRevisionNumber() {
		return revisionNumber;
	}

	public String getApplicationName() {
		return applicationName;
	}

	public String getInformationMail() {
		return informationMail;
	}

	public String getProviderPage() {
		return providerPage;
	}

	public String getProviderMail() {
		return providerMail;
	}

	public String getProviderTelephone() {
		return providerTelephone;
	}

	public String getBinaryFilesFolder() {
		return binaryFilesFolder;
	}
}