package cz.tendersystems.helpdesk.service.general;

import java.io.*;
import java.nio.file.*;
import java.text.*;
import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import org.apache.commons.io.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.entity.request.File;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class FileService {

	@PersistenceContext
	EntityManager em;

	@EJB
	ConfigurationService configurationService;

	@EJB
	RequestSecurityService requestSecurityService;

	@SuppressWarnings("unchecked")
	public List<FileData> loadFilesRequest(String idRequest) {
		if (!requestSecurityService.authorityLoadRequest(idRequest)) {
			throw new SecurityException("Nedostatecne opravneni nacist prilohy");
		}

		String HQL = "SELECT NEW cz.tendersystems.helpdesk.data.request.FileData(f) FROM File f LEFT JOIN f.comment c LEFT JOIN c.request r WHERE r.id = :idRequest ORDER BY f.id ASC";

		List<FileData> data = em.createQuery(HQL).setParameter("idRequest", idRequest).getResultList();

		return data;
	}

	public void saveFiles(List<FileData> metadatas, Comment comment) {
		for (FileData metadata : metadatas) {
			saveFile(metadata, comment);
		}
	}

	public void saveFile(FileData metadata, Comment comment) {
		if (!requestSecurityService.authorityLoadRequest(comment.getRequest().getId())) {
			throw new SecurityException("Nedostatecne opravneni vytvorit prilohu");
		}

		if (metadata == null) {
			throw new HelpdeskException("Binarni data nesmi byt prazdna");
		}
		if (metadata.getInputStream() == null) {
			throw new HelpdeskException("File input stream nesmi byt prazdny");
		}
		int inputStreamAvailable = 0;
		try {
			inputStreamAvailable = metadata.getInputStream().available();
		} catch (IOException e) {

		}
		if (inputStreamAvailable == 0) {
			throw new HelpdeskException("File input stream nesmi byt prazdny");
		}

		File file = new File(metadata);
		file.setCreatedDateTime(new Date());
		file.setComment(comment);
		em.persist(file);

		saveBinaryFile(file, metadata.getInputStream());
	}

	public FileData downloadFile(Long id) {
		try {
			FileData metadata = new FileData(findMetadata(id));
			metadata.setInputStream(new FileInputStream(configurationService.getBinaryFilesFolder() + prepareRelativePath(metadata.getCreatedDateTime()) + metadata.getId()));

			return metadata;
		} catch (FileNotFoundException e) {
			throw new HelpdeskException("Soubor nebyl nalezen");
		}
	}

	private File findMetadata(Long id) {
		File file = em.find(File.class, id);

		if (file == null) {
			throw new HelpdeskException("Soubor {0} neexistuje", id);
		}

		if (!requestSecurityService.authorityLoadRequest(file.getComment().getRequest().getId())) {
			throw new SecurityException("Nedostatecne opravneni stahnout soubor");
		}

		return file;
	}

	private void saveBinaryFile(File metadata, InputStream is) {
		String path = configurationService.getBinaryFilesFolder() + prepareRelativePath(metadata.getCreatedDateTime()) + metadata.getId();
		Path pathForDirs = Paths.get(path);

		try {
			Files.createDirectories(pathForDirs.getParent());
		} catch (IOException e) {
			throw new HelpdeskException("Nepovedlo se ulozit soubor");
		}

		if (pathForDirs.toFile().exists()) {
			throw new HelpdeskException("Soubor jiz existuje");
		}

		try {
			FileOutputStream out = new FileOutputStream(path);
			IOUtils.copy(is, out);
			out.close();
			is.close();
		} catch (IOException e) {
			throw new HelpdeskException("Nepodarilo se ulozit binarni data");
		}
	}

	private String prepareRelativePath(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return MessageFormat.format("/{0}/{1}/{2}/", Integer.toString(cal.get(Calendar.YEAR)), Integer.toString(cal.get(Calendar.MONTH) + 1), Integer.toString(cal.get(Calendar.DAY_OF_MONTH)));
	}

}
