package cz.tendersystems.helpdesk.service.general.mail;

import java.text.*;
import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.mail.*;
import javax.mail.Message.*;
import javax.mail.internet.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.service.configuration.*;

@Stateless
@LocalBean
public class MailService {

	private static final String contentTypeHtml = "text/html;charset=UTF-8";
	private static final String subjectPattern = "(?s).*<title>(.+?)</title>.*";

	@EJB
	MailTemplateService mailTemplateService;

	@EJB
	ConfigurationService configurationService;

	@Resource(mappedName = "java:jboss/mail/Default")
	private Session sessionMail;

	@Asynchronous
	public void sendMail(String templateName, String recipient, String... templateParameters) {
		prepareAndSendMail(templateName, Arrays.asList(recipient), templateParameters);
	}

	@Asynchronous
	public void sendMail(String templateName, List<String> recipients, String... templateParameters) {
		prepareAndSendMail(templateName, recipients, templateParameters);
	}

	private void prepareAndSendMail(String templateName, List<String> recipients, String... templateParameters) {
		String template = mailTemplateService.loadTemplate(templateName, templateParameters);

		template = mailTemplateService.addEmbedTemplate(EmbedTemplate.generalSalutationTemplate(configurationService.getVersionName()), template);
		template = mailTemplateService.addEmbedTemplate(EmbedTemplate.generalContactTemplate(configurationService.getProviderPage(), configurationService.getProviderTelephone(), configurationService.getProviderMail()), template);

		sendMail(template.replaceAll(subjectPattern, "$1"), template, contentTypeHtml, prepareRecipient(recipients));
	}

	private String prepareRecipient(List<String> recipients) {
		StringBuffer buffer = new StringBuffer();

		recipients.forEach(p -> buffer.append(MessageFormat.format("{0},", p)));

		buffer.deleteCharAt(buffer.length() - 1);

		return buffer.toString();
	}

	private void sendMail(String subject, String content, String contentType, String recipients) {
		write(LevelRecord.INFORMATION, "Odesilam email s predmetem [{0}] prijemcci [{1}]", subject, recipients);

		try {
			MimeMessage message = new MimeMessage(sessionMail);

			message.setSubject(subject);
			message.setContent(content, contentType);

			message.setFrom(new InternetAddress(configurationService.getInformationMail()));
			message.setRecipients(RecipientType.TO, recipients);

			message.setSentDate(new Date());

			Transport.send(message);
		} catch (Throwable e) {
			write(LevelRecord.ERROR, "Nepodarilo se odeslat email s predmetem {0} prijemci {1}", subject, recipients);

			throw new Error(e);
		}
	}

	private void write(LevelRecord levelRecord, String text, Object... parameters) {
		ToolsForLog.write(getClass(), levelRecord, text, parameters);
	}
}
