package cz.tendersystems.helpdesk.service.general.mail;

import java.io.*;
import java.nio.file.*;
import java.text.*;
import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.service.configuration.*;

@Singleton
@Startup
@LocalBean
public class MailTemplateService {

	private static final String fileNameWithTemplates = "mail-templates";
	private static final String fileFilter = "*.html";

	private Map<String, String> templates;

	@EJB
	ConfigurationService configurationService;

	@PostConstruct
	@Lock(LockType.WRITE)
	public void update() {
		try (DirectoryStream<Path> files = Files.newDirectoryStream(FileSystems.getDefault().getPath(Tools.getPathToServerFolder(fileNameWithTemplates) + Tools.FILE_PATH_SEPARATOR), fileFilter)) {
			write(LevelRecord.INFORMATION, "Nacitam sablony emailu");

			templates = new HashMap<>();

			for (Path p : files) {
				String name = p.getFileName().toString().split("\\.")[0];
				templates.put(name, new String(Files.readAllBytes(p), Tools.CHARSET));

				write(LevelRecord.INFORMATION, "Sablona {0} byla uspesne nactena", name);
			}

			templates = Collections.unmodifiableMap(templates);

			write(LevelRecord.INFORMATION, "Nacteni sablon emailu bylo uspesne dokonceno");
		} catch (IOException e) {
			templates = null;
			throw new HelpdeskException("Vyskytl se problem pri nacitani sablon emailu", e);
		}
	}

	@Lock(LockType.READ)
	public String loadTemplate(String name, String... parameters) {
		if (!templates.containsKey(name)) {
			throw new HelpdeskException("Sablona emailu s nazvem {0} neexistuje", name);
		}

		return new MessageFormat(templates.get(name), Tools.LOCALIZATION_OF_FORMATTER).format(parameters);
	}

	public String addEmbedTemplate(EmbedTemplate embedTemplate, String template) {
		return template.replace(embedTemplate.getSupplantStringValue(), MessageFormat.format(loadTemplate(embedTemplate.getTemplateName()), (Object[]) embedTemplate.getParameters()));
	}

	private void write(LevelRecord uroven, String text, Object... parametry) {
		ToolsForLog.write(getClass(), uroven, text, parametry);
	}

	public Map<String, String> getTemplates() {
		return templates;
	}
}
