package cz.tendersystems.helpdesk.service.list;

import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.persistence.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.service.security.*;

public abstract class AbstractListService<E, D, F extends Filter<E>> {

	@PersistenceContext
	EntityManager em;

	@Resource
	SessionContext context;

	@EJB
	protected BasicSecurityService basicSecurityService;

	public Response<D> find(RequestList<F> request) {
		CriteriaBuilder builder = em.getCriteriaBuilder();

		Long recordCounter = loadRecordCounter(builder, request);
		List<D> data = loadData(builder, request);

		return new Response<D>(data, recordCounter);
	}

	protected Long loadRecordCounter(CriteriaBuilder builder, RequestList<F> request) {
		CriteriaQuery<Long> query = builder.createQuery(Long.class);
		Root<E> root = query.from(loadEntityType());

		query.select(builder.countDistinct(root));
		processPredicate(builder, query, root, request);

		return em.createQuery(query).getSingleResult();
	}

	protected List<D> loadData(CriteriaBuilder builder, RequestList<F> request) {
		CriteriaQuery<E> query = builder.createQuery(loadEntityType());
		Root<E> root = query.from(loadEntityType());

		query.select(root);
		processPredicate(builder, query, root, request);
		processMinted(builder, query, root);

		return convert(em.createQuery(query).setFirstResult(request.getFirstIndexRecord()).setMaxResults(request.getRecordCounter()).getResultList());
	}

	protected Predicate preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<E> root) {
		return null;
	}

	protected Order prepareMinted(CriteriaBuilder builder, CriteriaQuery<?> query, Root<E> root) {
		return null;
	}

	private void processPredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<E> root, RequestList<F> request) {
		List<Predicate> predicates = request.getFilter().preparePredicate(builder, query, root);
		Predicate predicate = preparePredicate(builder, query, root);

		if (predicate != null) {
			predicates.add(predicate);
		}

		if (!predicates.isEmpty()) {
			query.where(predicates.toArray(new Predicate[predicates.size()]));
		}
	}

	private void processMinted(CriteriaBuilder builder, CriteriaQuery<?> query, Root<E> root) {
		Order minted = prepareMinted(builder, query, root);

		if (minted != null) {
			query.orderBy(minted);
		}
	}

	private List<D> convert(List<E> entity) {
		List<D> data = new ArrayList<D>();

		for (E entita : entity) {
			data.add(convert(entita));
		}

		return data;
	}

	protected abstract Class<E> loadEntityType();

	protected abstract D convert(E entity);
}