package cz.tendersystems.helpdesk.service.log;

import java.text.*;
import javax.ejb.*;
import javax.persistence.*;
import org.jboss.logging.*;
import cz.tendersystems.helpdesk.data.general.*;
import cz.tendersystems.helpdesk.entity.general.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class ApplicationLogService {

	@PersistenceContext
	EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	public void createLog(Class<?> rootClass, String operation, String details, Object... parameters) {
		saveLog(rootClass, new ApplicationLogData(operation, details, parameters));
	}

	public void createLog(Class<?> rootClass, ApplicationLogData data) {
		saveLog(rootClass, data);
	}

	private void saveLog(Class<?> rootClass, ApplicationLogData data) {
		ApplicationLog applicationLog = new ApplicationLog(data);

		if (basicSecurityService.isUserLogged()) {
			applicationLog.setLoginName(basicSecurityService.getLoginName());
		} else if (data.getLoginName() != null && !data.getLoginName().trim().isEmpty()) {
			applicationLog.setLoginName(data.getLoginName());
		}

		em.persist(applicationLog);

		logging(rootClass, applicationLog);
	}

	private void logging(Class<?> rootClass, ApplicationLog applicationLog) {
		Logger log = Logger.getLogger(rootClass);

		switch (applicationLog.getLevel()) {
			case INFORMATION:
				log.info(prepareMessage(applicationLog));
				break;
			case ERROR:
				log.error(prepareMessage(applicationLog));
				break;
			default:
				break;
		}
	}

	private String prepareMessage(ApplicationLog log) {
		StringBuilder zprava = new StringBuilder();

		if (log.getId() != null) {
			zprava.append(MessageFormat.format("Aplikacni log s ID {0}", log.getId()));
		} else {
			zprava.append("Aplikacni log");
		}

		zprava.append(MessageFormat.format(" pro ukon {0}", log.getOperation()));

		if (log.getIdRequest() != null) {
			zprava.append(MessageFormat.format(" u požadavku s ID {0}", log.getIdRequest()));
		}

		if (log.getLoginName() != null) {
			zprava.append(MessageFormat.format(" pro uĹľivatele {0}", log.getLoginName()));
		}

		zprava.append(". ");

		if (log.getDetails() != null) {
			zprava.append(log.getDetails());
		}

		return zprava.toString();
	}
}
