package cz.tendersystems.helpdesk.service.log.list;

import java.util.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.general.*;
import cz.tendersystems.helpdesk.service.list.*;

public class ApplicationLogFilter implements Filter<ApplicationLog> {

	private Long id;
	private Date byDate;
	private Date toDate;
	private String user;
	private String operation;
	private String details;
	private String idRequest;
	private LevelRecord level;

	public ApplicationLogFilter() {
	}

	@Override
	public List<Predicate> preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<ApplicationLog> root) {
		List<Predicate> predicates = new ArrayList<Predicate>();

		if (getId() != null) {
			predicates.add(builder.like(builder.lower(builder.function("str", String.class, root.<Long> get("id"))), "%" + getId() + "%"));
		}

		if (getOperation() != null) {
			predicates.add(builder.like(root.<String> get("operation"), "%" + getOperation() + "%"));
		}

		if (getByDate() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("createdDateTime"), getByDate()));
		}

		if (getToDate() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("createdDateTime"), getToDate()));
		}

		if (getDetails() != null) {
			predicates.add(builder.like(root.<String> get("details"), "%" + getDetails() + "%"));
		}

		if (getUser() != null) {
			predicates.add(builder.like(root.<String> get("loginName"), "%" + getUser() + "%"));
		}

		if (getIdRequest() != null) {
			predicates.add(builder.like(root.<String> get("idRequest"), "%" + getIdRequest() + "%"));
		}

		if (getLevel() != null) {
			predicates.add(builder.equal(root.<LevelRecord> get("level"), getLevel()));
		}

		return predicates;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Date getByDate() {
		return byDate;
	}

	public void setByDate(Date byDate) {
		this.byDate = byDate;
	}

	public Date getToDate() {
		return toDate;
	}

	public void setToDate(Date toDate) {
		this.toDate = toDate;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getOperation() {
		return operation;
	}

	public void setOperation(String operation) {
		this.operation = operation;
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		this.details = details;
	}

	public String getIdRequest() {
		return idRequest;
	}

	public void setIdRequest(String idRequest) {
		this.idRequest = idRequest;
	}

	public LevelRecord getLevel() {
		return level;
	}

	public void setLevel(LevelRecord level) {
		this.level = level;
	}
}