package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class OutsourcerService {

	@PersistenceContext
	private EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	public Outsourcer findOutsourcer(Long id) {
		if (!basicSecurityService.isUserLogged()) {
			throw new SecurityException("Nedostatecne opravneni pripravit zadavatele");
		}

		Outsourcer outsourcer = em.find(Outsourcer.class, id);

		if (outsourcer == null) {
			throw new HelpdeskException("Zadavatel {0} neexistuje", id);
		}

		return outsourcer;
	}

	public OutsourcerData loadOutsourcer(Long id) {
		return new OutsourcerData(findOutsourcer(id));
	}

	public Long createOutsourcer(OutsourcerData data) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni vytvorit zadavatele");
		}

		Outsourcer outsourcer = new Outsourcer(data);

		em.persist(outsourcer);

		applicationLogService.createLog(Outsourcer.class, "Vytvoření zadavatele", "Vytvoření zadavatele s id {0}", outsourcer.getId());

		return outsourcer.getId();
	}

	public void updateOutsourcer(OutsourcerData data) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit zadavatele");
		}

		Outsourcer outsourcer = findOutsourcer(data.getId());
		Outsourcer.convert(outsourcer, data);

		applicationLogService.createLog(Outsourcer.class, "Upravení zadavatele", "Upravení zadavatele s id {0}", outsourcer.getId());
	}

	@SuppressWarnings("unchecked")
	public void updateActivityOfOutsourcerAndUsers(Long idOutsourcer, boolean active) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit aktivitu zadavatele");
		}

		Outsourcer outsourcer = findOutsourcer(idOutsourcer);
		outsourcer.setActive(active);

		em.persist(outsourcer);

		String HQL = "SELECT u FROM UserOutsourcer u LEFT JOIN FETCH u.outsourcer o WHERE o.id = :idOutsourcer";
		List<UserOutsourcer> users = em.createQuery(HQL).setParameter("idOutsourcer", idOutsourcer).getResultList();

		users.forEach(u -> u.setUserAccountStatus(active ? UserAccountStatus.ENABLED : UserAccountStatus.FORBIDDEN));

		applicationLogService.createLog(Outsourcer.class, "Změna aktivity zadavatele", "Změna aktivity zadavatele s id {0}", outsourcer.getId());
	}

	public long loadIdOutsourcerUser(String loginName) {
		String HQL = "SELECT o.id FROM UserOutsourcer u LEFT JOIN u.outsourcer o WHERE u.loginName = :loginName";

		return (Long) em.createQuery(HQL).setParameter("loginName", loginName).getSingleResult();
	}
}
