package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.persistence.*;
import org.jboss.logging.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.general.*;

@Singleton
@Startup
@LocalBean
public class TestOrganizationalStructureService {

	@PersistenceContext
	private EntityManager em;

	@SuppressWarnings("unchecked")
	@PostConstruct
	public void createOrganizationalStrucure() {

		final String HQL = "from SystemUser";

		List<SystemUser> user = em.createQuery(HQL).getResultList();

		if (user.isEmpty()) {
			createUsersSolver();
			createUsersOutsourcer();
		}
	}

	private void createUsersSolver() {
		Logger.getLogger(getClass()).infov("Zacatek vytvareni uzivatelu {0}", getClass().getSimpleName());

		createSolverWithProduct("Tendermarket", "resitelTm", "pozorovatelTm");
		createSolverWithProduct("Tenderarena", "resitelTa", "pozorovatelTa");
		createUserSolver("administrator", new ArrayList<>(), RoleType.ADMINISTRATOR);

		Logger.getLogger(getClass()).infov("Uspesne vytvareni uzivatelu {0}", getClass().getSimpleName());
	}

	private void createUsersOutsourcer() {
		createOutsourcerWithUser("mzp", "uzivatelMzp");
		createOutsourcerWithUser("vzp", "uzivatelVzp");

	}

	private void createSolverWithProduct(String name, String loginSolverName, String loginWatcherName) {
		Product product = new Product();
		product.setName(name);
		product.setActive(true);

		em.persist(product);

		List<Product> products = new ArrayList<>();
		products.add(product);

		createUserSolver(loginSolverName, products, RoleType.SOLVER);
		createUserSolver(loginWatcherName, products, RoleType.WATCHER);
	}

	private void createOutsourcerWithUser(String companyName, String loginName) {
		Outsourcer outsourcer = new Outsourcer();

		outsourcer.setActive(true);
		outsourcer.setCompanyName(companyName);
		outsourcer.setCreatedDateTime(new Date());

		em.persist(outsourcer);

		createOutsourcerUser(loginName, outsourcer);
	}

	private void createUserSolver(String loginName, List<Product> products, RoleType type) {
		UserSolver user = new UserSolver();

		user.setLoginName(loginName);
		user.setPassword(Tools.generateHash("pass1234", loginName));
		user.setCreatedDateTime(new Date());
		user.setRoleType(type);
		user.setUserAccountStatus(UserAccountStatus.ENABLED);

		user.setPerson(preparePerson());

		if (!products.isEmpty()) {
			user.setProducts(products);
		}

		em.persist(user);

		em.flush();
	}

	private void createOutsourcerUser(String loginName, Outsourcer outsourcer) {
		UserOutsourcer user = new UserOutsourcer();

		user.setLoginName(loginName);
		user.setPassword(Tools.generateHash("pass1234", loginName));
		user.setCreatedDateTime(new Date());
		user.setRoleType(RoleType.OUTSOURCER);
		user.setUserAccountStatus(UserAccountStatus.ENABLED);

		user.setPerson(preparePerson());
		user.setOutsourcer(outsourcer);

		em.persist(user);
	}

	private Person preparePerson() {
		Person person = new Person();
		person.setName("Zbyněk");
		person.setSurname("Richter");
		person.setMail("zbytos@gmail.com");

		return person;
	}
}
