package cz.tendersystems.helpdesk.service.organizationalstructure.list;

import java.util.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.list.*;

public class UserFilter implements Filter<SystemUser> {

	private String loginName;
	private String name;
	private String surname;
	private String mail;
	private Long telephone;
	private RoleType roleType;
	private UserAccountStatus userAccountStatus;
	private Date byDate;
	private Date toDate;

	public UserFilter() {
	}

	@Override
	public List<Predicate> preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<SystemUser> root) {
		List<Predicate> predicates = new ArrayList<Predicate>();

		if (getLoginName() != null) {
			predicates.add(builder.like(root.<String> get("loginName"), "%" + getLoginName() + "%"));
		}

		if (getName() != null) {
			predicates.add(builder.like(root.<String> get("person").get("name"), "%" + getName() + "%"));
		}

		if (getSurname() != null) {
			predicates.add(builder.like(root.<String> get("person").get("surname"), "%" + getSurname() + "%"));
		}

		if (getMail() != null) {
			predicates.add(builder.like(root.<String> get("person").get("mail"), "%" + getMail() + "%"));
		}

		if (getTelephone() != null) {
			predicates.add(builder.like(builder.lower(builder.function("str", String.class, root.<Long> get("person").get("telephone"))), "%" + getTelephone() + "%"));
		}

		if (getRoleType() != null) {
			predicates.add(builder.equal(root.<Boolean> get("roleType"), getRoleType()));
		}

		if (getUserAccountStatus() != null) {
			predicates.add(builder.equal(root.<Boolean> get("userAccountStatus"), getUserAccountStatus()));
		}

		if (getByDate() != null) {
			predicates.add(builder.greaterThanOrEqualTo(root.<Date> get("createdDateTime"), getByDate()));
		}

		if (getToDate() != null) {
			predicates.add(builder.lessThanOrEqualTo(root.<Date> get("createdDateTime"), getToDate()));
		}

		return predicates;
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getMail() {
		return mail;
	}

	public void setMail(String mail) {
		this.mail = mail;
	}

	public Long getTelephone() {
		return telephone;
	}

	public void setTelephone(Long telephone) {
		this.telephone = telephone;
	}

	public RoleType getRoleType() {
		return roleType;
	}

	public void setRoleType(RoleType roleType) {
		this.roleType = roleType;
	}

	public UserAccountStatus getUserAccountStatus() {
		return userAccountStatus;
	}

	public void setUserAccountStatus(UserAccountStatus userAccountStatus) {
		this.userAccountStatus = userAccountStatus;
	}

	public Date getByDate() {
		return byDate;
	}

	public void setByDate(Date byDate) {
		this.byDate = byDate;
	}

	public Date getToDate() {
		return toDate;
	}

	public void setToDate(Date toDate) {
		this.toDate = toDate;
	}
}