package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class ProductService {

	@PersistenceContext
	private EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	public Product findProduct(Long id) {
		Product product = em.find(Product.class, id);

		if (product == null) {
			throw new HelpdeskException("Produkt {0} neexistuje", id);
		}

		return product;
	}

	public ProductData loadProduct(Long id) {
		return new ProductData(findProduct(id));
	}

	public Long createProduct(ProductData data) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni vytvorit produkt");
		}

		Product product = new Product(data);

		em.persist(product);

		applicationLogService.createLog(Product.class, "Vytvoření produktu", "Vytvoření produktu s id {0}", product.getId());

		return product.getId();
	}

	public void updateProduct(ProductData data) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit produkt");
		}

		Product product = findProduct(data.getId());
		Product.convert(product, data);

		applicationLogService.createLog(Product.class, "Upravení produktu", "Upravení produktu s id {0}", product.getId());
	}

	public void updateActivityOfProduct(Long idProduct, boolean active) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit aktivitu produktu");
		}

		Product product = findProduct(idProduct);
		product.setActive(active);

		applicationLogService.createLog(Product.class, "Změna aktivity produktu", "Změna aktivity produktu s id {0}", product.getId());
	}

	@SuppressWarnings("unchecked")
	public List<Long> loadIdProductUser(String loginName) {
		String HQL = "SELECT p.id FROM UserSolver u LEFT JOIN u.products p WHERE u.loginName = :loginName";

		return em.createQuery(HQL).setParameter("loginName", loginName).getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<ProductData> loadProducts() {
		String HQL = "SELECT NEW cz.tendersystems.helpdesk.data.organizationalstructure.sla.ProductData(p) FROM Product p";

		return em.createQuery(HQL).getResultList();
	}
}
