package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class TimeActivityService {

	@PersistenceContext
	EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	ApplicationLogService applicationLogService;

	@SuppressWarnings("unchecked")
	private List<TimeActivity> findTimesActivityOfSlaSchema(Long idSlaSchema) {
		String HQL = "SELECT t FROM TimeActivity t LEFT JOIN FETCH t.slaSchema s WHERE s.id = :idSlaSchema";

		return em.createQuery(HQL).setParameter("idSlaSchema", idSlaSchema).getResultList();
	}

	public List<TimeActivityData> loadTimesActivityOfSlaSchema(Long idSlaSchema) {
		List<TimeActivityData> data = new ArrayList<>();
		findTimesActivityOfSlaSchema(idSlaSchema).forEach(d -> data.add(new TimeActivityData(d)));

		return data;
	}

	public void updateTimesActivity(List<TimeActivityData> data, SlaSchema slaSchema) {
		if (!basicSecurityService.isUserAdministrator()) {
			throw new SecurityException("Nedostatecne opravneni upravit doby aktivity");
		}
		List<TimeActivity> oldTimesActivity = slaSchema.getTimesActivity();
		List<TimeActivity> newTimeActivity = prepareNewTimesActivity(data, oldTimesActivity, slaSchema);

		List<TimeActivity> removedTimesActivity = new ArrayList<>(oldTimesActivity);
		removedTimesActivity.removeAll(newTimeActivity);

		List<TimeActivity> addedTimesActivity = new ArrayList<>(newTimeActivity);
		addedTimesActivity.removeAll(oldTimesActivity);

		removedTimesActivity.forEach(d -> em.remove(em.contains(d) ? d : em.merge(d)));
		addedTimesActivity.forEach(d -> em.persist(d));

		slaSchema.getTimesActivity().removeAll(removedTimesActivity);
		slaSchema.getTimesActivity().addAll(addedTimesActivity);

		applicationLogService.createLog(SlaSchema.class, "Upravení dob aktivit", "Upravení dob aktivit u sla schéma s id {0}", slaSchema.getId());
	}

	private List<TimeActivity> prepareNewTimesActivity(List<TimeActivityData> data, List<TimeActivity> oldTimesActivity, SlaSchema slaSchema) {
		Map<Long, TimeActivity> mapsOldTimesActivity = new HashMap<>();
		oldTimesActivity.forEach(d -> mapsOldTimesActivity.put(d.getId(), d));

		List<TimeActivity> timesActivity = new ArrayList<>();
		data.forEach(d -> timesActivity.add(prepareTimeActivity(d, mapsOldTimesActivity, slaSchema)));

		return timesActivity;
	}

	private TimeActivity prepareTimeActivity(TimeActivityData data, Map<Long, TimeActivity> mapsOldTimesActivity, SlaSchema slaSchema) {
		TimeActivity entity = data.getId() == null ? new TimeActivity(data) : mapsOldTimesActivity.get(data.getId());

		if (data.getId() == null) {
			entity.setSlaSchema(slaSchema);
		} else {
			TimeActivity.convert(entity, data);
		}

		return entity;
	}
}
