package cz.tendersystems.helpdesk.service.organizationalstructure.sla.list;

import java.util.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.list.*;

public class SlaSchemaFilter implements Filter<SlaSchema> {

	private Long id;
	private String name;
	private Boolean active;
	private ProductData product;

	public SlaSchemaFilter() {
	}

	@Override
	public List<Predicate> preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<SlaSchema> root) {
		List<Predicate> predicates = new ArrayList<Predicate>();

		if (getId() != null) {
			predicates.add(builder.like(builder.lower(builder.function("str", String.class, root.<Long> get("id"))), "%" + getId() + "%"));
		}

		if (getName() != null) {
			predicates.add(builder.like(root.<String> get("name"), "%" + getName() + "%"));
		}

		if (getActive() != null) {
			predicates.add(builder.equal(root.<Boolean> get("active"), getActive()));
		}

		if (getProduct() != null) {
			predicates.add(preparePredicateOnProduct(builder, query, root));
		}

		return predicates;
	}

	private Predicate preparePredicateOnProduct(CriteriaBuilder builder, CriteriaQuery<?> query, Root<SlaSchema> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Assignment> rootSubquery = subquery.from(Assignment.class);

		subquery.select(rootSubquery.<Long> get("id"));
		subquery.where(builder.like(builder.lower(builder.function("str", String.class, rootSubquery.join("product").<Long> get("id"))), "%" + getProduct().getId() + "%"));

		return root.<Long> get("id").in(subquery);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Boolean getActive() {
		return active;
	}

	public void setActive(Boolean active) {
		this.active = active;
	}

	public ProductData getProduct() {
		return product;
	}

	public void setProduct(ProductData product) {
		this.product = product;
	}
}