package cz.tendersystems.helpdesk.service.request;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.service.log.*;

@Singleton
@LocalBean
public class ArchivedRequestService {

	private final static Integer countDayForArchivedRequest = 365;

	@PersistenceContext
	EntityManager em;

	@EJB
	ApplicationLogService applicationLogService;

	@SuppressWarnings("unchecked")
	@Schedule(hour = "0", minute = "04", second = "0", persistent = false)
	public void changeRequestStatus() {
		final String hql = "SELECT DISTINCT(r) FROM Request r WHERE r.requestStatus = 'SOLVED' AND EXTRACT(DAY FROM (NOW() - r.dateOfClose)) > :countDayForArchivedRequest";
		Set<String> idArchivedRequest = new HashSet<>();

		for (Request r : (List<Request>) em.createQuery(hql).setParameter("countDayForArchivedRequest", countDayForArchivedRequest).getResultList()) {
			r.setRequestStatus(RequestStatus.ARCHIVED);
			idArchivedRequest.add(r.getId());
		}

		if (!idArchivedRequest.isEmpty()) {
			applicationLogService.createLog(Request.class, "Změna stavu požadavku", "U požadavku s ID {0} byl změněn stav na Archívován.", idArchivedRequest);
		}
	}
}
