package cz.tendersystems.helpdesk.service.request.list;

import java.util.*;
import javax.ejb.*;
import javax.persistence.criteria.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.security.*;

@Stateless
@LocalBean
public class RequestListService extends AbstractListService<Request, RequestData, RequestFilter> {

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	OutsourcerService outsourcerService;

	@EJB
	ProductService productService;

	@Override
	protected Class<Request> loadEntityType() {
		return Request.class;
	}

	@Override
	protected RequestData convert(Request entity) {
		return new RequestData(entity);
	}

	@Override
	protected Order prepareMinted(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		return builder.desc(root.<Date> get("createdDateTime"));
	}

	@Override
	public Response<RequestData> find(RequestList<RequestFilter> request) {
		if (!basicSecurityService.isUserLogged()) {
			throw new SecurityException("Nedostatecne opravneni nacist seznam pozadavku");
		}

		return super.find(request);
	}

	@Override
	protected Predicate preparePredicate(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		if (basicSecurityService.isRoleType(RoleType.ADMINISTRATOR)) {
			return null;
		}

		if (basicSecurityService.isRoleType(RoleType.OUTSOURCER)) {
			return preparePredicateForOutsourcer(builder, query, root);
		}

		if (basicSecurityService.isRoleType(RoleType.SOLVER) || basicSecurityService.isRoleType(RoleType.WATCHER)) {
			return preparePredicateForSolver(builder, query, root);
		}

		return super.preparePredicate(builder, query, root);
	}

	private Predicate preparePredicateForOutsourcer(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Outsourcer> rootSubquery = subquery.from(Outsourcer.class);

		subquery.select(rootSubquery.<Long> get("id"));
		subquery.where(builder.like(builder.lower(builder.function("str", String.class, rootSubquery.<Long> get("id"))), "%" + outsourcerService.loadIdOutsourcerUser(basicSecurityService.getLoginName()) + "%"));

		return root.join("author").join("loggedUser").join("outsourcer").<String> get("id").in(subquery);
	}

	// TODO produkty s uživatele
	private Predicate preparePredicateForSolver(CriteriaBuilder builder, CriteriaQuery<?> query, Root<Request> root) {
		Subquery<Long> subquery = query.subquery(Long.class);
		Root<Assignment> rootSubquery = subquery.from(Assignment.class);

		subquery.select(rootSubquery.join("slaSchema").<Long> get("id"));
		subquery.where(rootSubquery.join("product").get("id").in(productService.loadIdProductUser(basicSecurityService.getLoginName())));

		return root.join("slaSchema").<Long> get("id").in(subquery);
	}
}
