package cz.tendersystems.helpdesk.service.security;

import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@Stateless
@LocalBean
public class RequestSecurityService {

	@PersistenceContext
	EntityManager em;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	UserService userService;

	@EJB
	SlaSchemaService slaSchemaService;

	public boolean authorityLoadRequest(String idRequest) {
		Request request = em.find(Request.class, idRequest);

		if (request.getAuthor().getAnonymous() != null) {
			return true;
		}

		if (request.getAuthor().getAnonymous() == null && !basicSecurityService.isUserLogged()) {
			return false;
		}

		return preparePredicateForLoggedUser(request);
	}

	public boolean authorityCreateComment(String idRequest) {
		return !RequestStatus.ARCHIVED.equals(em.find(Request.class, idRequest).getRequestStatus()) && authorityLoadRequest(idRequest);
	}

	public boolean authorityUpdateRequest(String idRequest, RequestStatus status) {
		Request request = em.find(Request.class, idRequest);

		if (!basicSecurityService.isUserLogged() || basicSecurityService.isRoleType(RoleType.OUTSOURCER) || basicSecurityService.isRoleType(RoleType.WATCHER) || !predicateOnRequestStatus(request, status)) {
			return false;
		}

		return authorityLoadRequest(idRequest);
	}

	private boolean predicateOnRequestStatus(Request request, RequestStatus status) {
		return !status.equals(request.getRequestStatus()) && !RequestStatus.ARCHIVED.equals(request.getRequestStatus()) && !RequestStatus.SOLVED.equals(request.getRequestStatus()) && !(RequestStatus.NEW.equals(request.getRequestStatus()) && status.equals(RequestStatus.SOLVED));
	}

	private boolean preparePredicateForLoggedUser(Request request) {
		SystemUser user = userService.findUser(basicSecurityService.getLoginName());

		switch (user.getRoleType()) {
			case ADMINISTRATOR:
				return true;

			case SOLVER:
				return preparePredicateForSolver(request.getSlaSchema().getId(), ((UserSolver) user).getProducts());

			case WATCHER:
				return preparePredicateForSolver(request.getSlaSchema().getId(), ((UserSolver) user).getProducts());

			case OUTSOURCER:
				return request.getAuthor().getLoggedUser() == null ? false : preparePredicateForOutsourcer((UserOutsourcer) user, request.getAuthor().getLoggedUser().getOutsourcer());

			default:
				return false;
		}
	}

	private boolean preparePredicateForSolver(Long idSlaSchema, List<Product> products) {
		AssignmentData assignments = slaSchemaService.loadAssignmentSlaSchema(idSlaSchema);

		for (Product product : products) {
			if (assignments.getProduct().getId().equals(product.getId())) {
				return true;
			}
		}

		return false;
	}

	private boolean preparePredicateForOutsourcer(UserOutsourcer user, Outsourcer outsourcer) {
		return user.getOutsourcer().equals(outsourcer);
	}
}
