package cz.tendersystems.helpdesk;

import java.text.*;
import java.util.*;
import javax.inject.*;
import javax.persistence.*;
import javax.security.auth.login.*;
import javax.transaction.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.asset.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.general.*;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.service.log.*;
import cz.tendersystems.helpdesk.service.security.*;

public abstract class AbstractEJBTest {

	protected final static Long ID_NOT_EXIST_OBJECT = 0l;
	protected final static String LOGIN_NAME_ADMINISTRATOR = "administrator";
	protected final static String SOLVER_LOGIN_NAME = "resitel";

	@PersistenceContext
	protected EntityManager em;

	@Inject
	protected UserTransaction transaction;

	private LoginContext loginContext;

	public static JavaArchive prepareArchive() {
		JavaArchive jar = ShrinkWrap.create(JavaArchive.class, "helpdesk-test.jar");
		jar.addClass(AbstractEJBTest.class);
		jar.addPackages(true, "cz.tendersystems.helpdesk.data");
		jar.addPackages(true, "cz.tendersystems.helpdesk.entity");
		jar.addPackages(true, "cz.tendersystems.helpdesk.exception");
		jar.addClasses(ApplicationLogService.class, ToolsForLog.class, Tools.class, BasicSecurityService.class, ArquillianAuthentic.class);
		jar.addAsResource("cz/tendersystems/helpdesk/resources/messages.properties");
		jar.addAsResource("META-INF/jboss-ejb.xml");
		jar.addAsResource("META-INF/ejb-jar.xml");
		jar.addAsResource("users.properties");
		jar.addAsResource("roles.properties");
		jar.addAsManifestResource(EmptyAsset.INSTANCE, "beans.xml");
		jar.addAsManifestResource("test-persistence.xml", "persistence.xml");

		return jar;
	}

	protected void login(String loginName) {
		final String password = "pass1234";

		try {
			loginContext = ArquillianAuthentic.createLoginContext(loginName, password);
			loginContext.login();
		} catch (LoginException e) {
			e.printStackTrace();
		}
	}

	@After
	public void logout() {
		try {
			if (loginContext != null) {
				loginContext.logout();
			}
		} catch (LoginException e) {
			e.printStackTrace();
		}
	}

	protected Date prepareDate() {
		return prepareDate("2016-02-22 00.00.00");
	}

	protected Date prepareDate(String date) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh.mm.ss");
		String dateInString = date;
		try {
			return sdf.parse(dateInString);
		} catch (ParseException e) {
			e.printStackTrace();
		}

		return null;
	}

	@SuppressWarnings("unchecked")
	protected void checkApplicationLog(String operation, String details, String loginName, String idRequest) {
		List<ApplicationLog> logs = em.createQuery("SELECT a FROM ApplicationLog a WHERE a.details = :details").setParameter("details", details).getResultList();

		for (ApplicationLog applicationLog : logs) {
			Assert.assertEquals(operation, applicationLog.getOperation());

			if (details != null) {
				Assert.assertEquals(details, applicationLog.getDetails());
			}

			Assert.assertNotNull(applicationLog.getCreateDateTime());

			if (loginName != null) {
				Assert.assertEquals(loginName, applicationLog.getLoginName());
			} else {
				Assert.assertNull(applicationLog.getLoginName());
			}

			if (idRequest != null) {
				Assert.assertEquals(idRequest, applicationLog.getIdRequest());
			} else {
				Assert.assertNull(applicationLog.getIdRequest());
			}

			Assert.assertEquals(LevelRecord.INFORMATION, applicationLog.getLevel());
		}

	}
}