package cz.tendersystems.helpdesk;

import java.io.*;
import java.util.*;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;

public class ArquillianAuthentic {

	static class NamePasswordCallbackHandler implements CallbackHandler {

		private final String loginName;
		private final String password;

		private NamePasswordCallbackHandler(String loginName, String password) {
			this.loginName = loginName;
			this.password = password;
		}

		public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
			for (Callback current : callbacks) {
				if (current instanceof NameCallback) {
					((NameCallback) current).setName(loginName);
				} else if (current instanceof PasswordCallback) {
					((PasswordCallback) current).setPassword(password.toCharArray());
				} else {
					throw new UnsupportedCallbackException(current);
				}
			}
		}
	}

	static class JBossJaasConfiguration extends Configuration {

		private final String configurationName;

		JBossJaasConfiguration(String configurationName) {
			this.configurationName = configurationName;
		}

		@Override
		public AppConfigurationEntry[] getAppConfigurationEntry(String name) {
			if (!configurationName.equals(name)) {
				throw new IllegalArgumentException("Neocekavana chyba konfigurace '" + name + "'");
			}

			return new AppConfigurationEntry[] {

					createUsersRolesLoginModuleConfigEntry(),

					createClientLoginModuleConfigEntry(),

			};
		}

		private AppConfigurationEntry createUsersRolesLoginModuleConfigEntry() {
			Map<String, String> options = new HashMap<String, String>();
			return new AppConfigurationEntry("org.jboss.security.auth.spi.UsersRolesLoginModule", AppConfigurationEntry.LoginModuleControlFlag.REQUIRED, options);
		}

		private AppConfigurationEntry createClientLoginModuleConfigEntry() {
			Map<String, String> options = new HashMap<String, String>();
			options.put("multi-threaded", "true");
			options.put("restore-login-identity", "true");

			return new AppConfigurationEntry("org.jboss.security.ClientLoginModule", AppConfigurationEntry.LoginModuleControlFlag.REQUIRED, options);
		}
	}

	public static LoginContext createLoginContext(final String loginName, final String password) throws LoginException {
		final String configurationName = "Prihlaseni pomoc Arquillianu";

		CallbackHandler cbh = new ArquillianAuthentic.NamePasswordCallbackHandler(loginName, password);
		Configuration config = new JBossJaasConfiguration(configurationName);

		return new LoginContext(configurationName, new Subject(), cbh, config);
	}

}
