package cz.tendersystems.helpdesk.data.request;

import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;

public class RequestDataTest {

	@Test
	public void shouldCorrectConstuctor() {
		RequestData request = new RequestData(prepareRequest());

		Assert.assertEquals("894orf9hdwhewashaod5j8vz4ieafuooi1d7ip5pabq4a2sbim", request.getId());
		Assert.assertEquals("Test požadavku", request.getName());
		Assert.assertNotNull(request.getCreatedDateTime());
		Assert.assertNotNull(request.getReactionTime());
		Assert.assertNotNull(request.getTimeToSolve());
		Assert.assertNotNull(request.getDateOfClose());
		Assert.assertNotNull(request.getSlaSchema());
		Assert.assertNotNull(request.getAuthor());
		Assert.assertNotNull(request.getSolver());
	}

	@Test
	public void shouldReturnSameEquals() {
		Assert.assertTrue(prepareRequestData().equals(prepareRequestData()));
	}

	@Test
	public void shouldReturnSameObject() {
		Assert.assertTrue(prepareRequestData().toString().equals(prepareRequestData().toString()));
	}

	@Test
	public void shouldReturnSameHashCode() {
		Assert.assertEquals(prepareRequestData().hashCode(), prepareRequestData().hashCode());
	}

	private Request prepareRequest() {
		Request request = new Request();
		request.setId("894orf9hdwhewashaod5j8vz4ieafuooi1d7ip5pabq4a2sbim");
		request.setName("Test požadavku");
		request.setCreatedDateTime(new Date());
		request.setReactionTime(new Date());
		request.setTimeToSolve(new Date());
		request.setDateOfClose(new Date());
		request.setSlaSchema(new SlaSchema());
		request.setAuthor(prepareAuthor());
		request.setSolver(prepareUserSolver());

		return request;
	}

	private RequestData prepareRequestData() {
		return new RequestData(prepareRequest());
	}

	private Author prepareAuthor() {
		Author author = new Author();

		author.setId(1l);
		author.setCompanyName("tendersystems");
		author.setAnonymous(new Person());

		author.setLoggedUser(new UserOutsourcer());
		author.getLoggedUser().setPerson(new Person());
		author.getLoggedUser().setOutsourcer(new Outsourcer());
		author.getLoggedUser().setChangePassword(new ChangePassword());

		return author;
	}

	private UserSolver prepareUserSolver() {
		UserSolver user = new UserSolver();
		user.setLoginName("zbynek");
		user.setCreatedDateTime(new Date());
		user.setRoleType(RoleType.WATCHER);
		user.setPerson(new Person());
		user.setProducts(Arrays.asList(new Product()));

		return user;
	}
}
