package cz.tendersystems.helpdesk.entity.request;

import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;

public class CommentTest {

	@Test
	public void shouldCorrectConstuctor() {
		Comment comment = prepareComment();

		Assert.assertNotNull(comment.getId());
		Assert.assertEquals("Test komentáře", comment.getText());
		Assert.assertNotNull(comment.getCreatedDateTime());
		Assert.assertEquals(RequestStatus.NEW, comment.getRequestStatus());
		Assert.assertEquals("Zbyněk Richter", comment.getAuthor());
		Assert.assertTrue(comment.getAttachments().isEmpty());
		Assert.assertNotNull(comment.getRequest());
	}

	@Test
	public void shouldConvertObject() {
		Comment comment = new Comment();

		Comment.convert(comment, prepareCommentData());

		Assert.assertNull(comment.getId());
		Assert.assertEquals("Test", comment.getText());
		Assert.assertNotNull(comment.getCreatedDateTime());
		Assert.assertEquals(RequestStatus.SOLVED, comment.getRequestStatus());
		Assert.assertEquals("Richter", comment.getAuthor());
		Assert.assertNull(comment.getAttachments());
		Assert.assertNull(comment.getRequest());
	}

	@Test
	public void shouldReturnSameEquals() {
		Assert.assertTrue(prepareComment().equals(prepareComment()));
	}

	@Test
	public void shouldReturnSameObject() {
		Assert.assertTrue(prepareComment().toString().equals(prepareComment().toString()));
	}

	@Test
	public void shouldReturnSameHashCode() {
		Assert.assertEquals(prepareComment().hashCode(), prepareComment().hashCode());
	}

	private Comment prepareComment() {
		Comment comment = new Comment();

		comment.setId(1l);
		comment.setText("Test komentáře");
		comment.setCreatedDateTime(new Date());
		comment.setRequestStatus(RequestStatus.NEW);
		comment.setAuthor("Zbyněk Richter");
		comment.setAttachments(new ArrayList<>());
		comment.setRequest(new Request());

		return comment;
	}

	private CommentData prepareCommentData() {
		CommentData comment = new CommentData();

		comment.setId(2l);
		comment.setText("Test");
		comment.setCreatedDateTime(new Date());
		comment.setRequestStatus(RequestStatus.SOLVED);
		comment.setAuthor("Richter");

		return comment;
	}
}
