package cz.tendersystems.helpdesk.entity.request;

import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;

public class RequestTest {

	@Test
	public void shouldCorrectConstuctor() {
		Request request = prepareRequest();

		Assert.assertEquals("894orf9hdwhewashaod5j8vz4ieafuooi1d7ip5pabq4a2sbim", request.getId());
		Assert.assertEquals("Test požadavku", request.getName());
		Assert.assertNotNull(request.getCreatedDateTime());
		Assert.assertNotNull(request.getReactionTime());
		Assert.assertNotNull(request.getTimeToSolve());
		Assert.assertNotNull(request.getDateOfClose());
		Assert.assertNotNull(request.getSlaSchema());
		Assert.assertNotNull(request.getAuthor());
		Assert.assertNotNull(request.getSolver());
		Assert.assertTrue(request.getComments().isEmpty());
	}

	@Test
	public void shouldConvertObject() {
		Request request = new Request();

		Request.convert(request, prepareRequestData());

		Assert.assertEquals("1", request.getId());
		Assert.assertEquals("Test požadavku 2", request.getName());
		Assert.assertNotNull(request.getCreatedDateTime());
		Assert.assertNotNull(request.getReactionTime());
		Assert.assertNotNull(request.getTimeToSolve());
		Assert.assertNotNull(request.getDateOfClose());
		Assert.assertNotNull(request.getSlaSchema());
		Assert.assertNotNull(request.getAuthor());
		Assert.assertNotNull(request.getSolver());
		Assert.assertNull(request.getComments());
	}

	@Test
	public void shouldReturnSameEquals() {
		Assert.assertTrue(prepareRequest().equals(prepareRequest()));
	}

	@Test
	public void shouldReturnSameObject() {
		Assert.assertTrue(prepareRequest().toString().equals(prepareRequest().toString()));
	}

	@Test
	public void shouldReturnSameHashCode() {
		Assert.assertEquals(prepareRequest().hashCode(), prepareRequest().hashCode());
	}

	private Request prepareRequest() {
		Request request = new Request();
		request.setId("894orf9hdwhewashaod5j8vz4ieafuooi1d7ip5pabq4a2sbim");
		request.setName("Test požadavku");
		request.setCreatedDateTime(new Date());
		request.setReactionTime(new Date());
		request.setTimeToSolve(new Date());
		request.setDateOfClose(new Date());
		request.setSlaSchema(new SlaSchema());
		request.setAuthor(prepareAuthor());
		request.setSolver(prepareUserSolver());
		request.setComments(new ArrayList<>());

		return request;
	}

	private RequestData prepareRequestData() {
		RequestData request = new RequestData();
		request.setId("1");
		request.setName("Test požadavku 2");
		request.setCreatedDateTime(new Date());
		request.setReactionTime(new Date());
		request.setTimeToSolve(new Date());
		request.setDateOfClose(new Date());
		request.setSlaSchema(new SlaSchemaData());
		request.setAuthor(new AuthorData(prepareAuthor()));
		request.setSolver(new UserSolverData(prepareUserSolver()));

		return request;
	}

	private Author prepareAuthor() {
		Author author = new Author();

		author.setId(1l);
		author.setCompanyName("tendersystems");
		author.setAnonymous(new Person());

		author.setLoggedUser(new UserOutsourcer());
		author.getLoggedUser().setPerson(new Person());
		author.getLoggedUser().setOutsourcer(new Outsourcer());
		author.getLoggedUser().setChangePassword(new ChangePassword());

		return author;
	}

	private UserSolver prepareUserSolver() {
		UserSolver user = new UserSolver();
		user.setLoginName("zbynek");
		user.setCreatedDateTime(new Date());
		user.setRoleType(RoleType.WATCHER);
		user.setPerson(new Person());
		user.setProducts(Arrays.asList(new Product()));

		return user;
	}
}
