package cz.tendersystems.helpdesk.general;

import java.text.*;
import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.request.*;

public class ToolsForRequestDatesTest {

	@Test
	public void shouldEquelsDates1() {
		ToolsForRequestDates tools = new ToolsForRequestDates();
		tools.calculateRequestDates(prepareRequest("12.06.2016 13:00:00"));
		Assert.assertEquals("Thu Jun 16 09:00:00 CEST 2016", tools.getReactionTime().toString());
		Assert.assertEquals("Thu Jun 16 11:00:00 CEST 2016", tools.getTimeToSolve().toString());
	}

	@Test
	public void shouldEquelsDates2() {
		ToolsForRequestDates tools = new ToolsForRequestDates();
		tools.calculateRequestDates(prepareRequest("25.03.2016 13:00:00"));
		Assert.assertEquals("Thu Mar 31 09:00:00 CEST 2016", tools.getReactionTime().toString());
		Assert.assertEquals("Thu Mar 31 11:00:00 CEST 2016", tools.getTimeToSolve().toString());
	}

	@Test
	public void shouldEquelsDates3() {
		ToolsForRequestDates tools = new ToolsForRequestDates();
		tools.calculateRequestDates(prepareRequest("11.08.2016 13:00:00"));
		Assert.assertEquals("Thu Aug 11 14:00:00 CEST 2016", tools.getReactionTime().toString());
		Assert.assertEquals("Thu Aug 11 16:00:00 CEST 2016", tools.getTimeToSolve().toString());
	}

	private Request prepareRequest(String createdDateTime) {
		Request request = new Request();

		request.setCreatedDateTime(prepareDate(createdDateTime, DateAndTimeFormat.FORMAT_OF_DATE_AND_TIME));
		request.setSlaSchema(prepareSlaSchema());

		return request;
	}

	private SlaSchema prepareSlaSchema() {
		SlaSchema slaSchema = new SlaSchema();

		slaSchema.setReactionTime(prepareDate("01:00:00", DateAndTimeFormat.FORMAT_OF_TIME));
		slaSchema.setTimeToSolve(prepareDate("02:00:00", DateAndTimeFormat.FORMAT_OF_TIME));
		slaSchema.setTimesActivity(prepareTimesActivity());

		return slaSchema;
	}

	private List<TimeActivity> prepareTimesActivity() {
		List<TimeActivity> timesActivity = new ArrayList<>();

		TimeActivity timeActivity = new TimeActivity();
		timeActivity.setDayName(DayName.THURSDAY);
		timeActivity.setStartTime(prepareDate("08:00:00", DateAndTimeFormat.FORMAT_OF_TIME));
		timeActivity.setEndTime(prepareDate("16:00:00", DateAndTimeFormat.FORMAT_OF_TIME));

		timesActivity.add(timeActivity);

		return timesActivity;
	}

	private Date prepareDate(String date, DateAndTimeFormat format) {
		SimpleDateFormat sdf = new SimpleDateFormat(format.getFormat());
		String dateInString = date;
		try {
			return sdf.parse(dateInString);
		} catch (ParseException e) {
			e.printStackTrace();
		}

		return null;
	}

}
