package cz.tendersystems.helpdesk.general;

import java.text.*;
import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;

public class ToolsTest {

	@Test
	public void shouldReturnConstante() throws ParseException {
		Assert.assertEquals("/", Tools.FILE_PATH_SEPARATOR);
		Assert.assertEquals("UTF-8", Tools.CHARSET);
		Assert.assertEquals("cs_CZ", Tools.LOCALIZATION);
		Assert.assertEquals(new Locale("cs", "CZ"), Tools.LOCALIZATION_OF_FORMATTER);
		Assert.assertEquals(new Long(20), Tools.FORMATTER_OF_WHOLE_NUMBERS.parse("20"));
		Assert.assertEquals(new Double("11.12"), Tools.FORMATTER_OF_FILE_SIZE.parse("11,12"));
	}

	@Test
	public void shouldPrepareEntityOfRightType() {
		Assert.assertEquals(SlaSchema.class, Tools.prepareEntityOfRightType(SlaSchema.class));
	}

	@Test
	public void shouldGenerateRandomString() {
		Assert.assertEquals(10, Tools.generateRandomString(10).length());
	}

	@Test
	public void shouldGenerateHash() {
		Assert.assertEquals("b0b6a5c202ff8e8ed90a6319cc020065a6ead513", Tools.generateHash("zbynek", "test"));
	}

	@Test
	public void shouldGenerateLink() {
		Map<String, String> parameters = new HashMap<>();
		parameters.put("id", "1");

		Assert.assertEquals("www.tenderdesk.cz/detail.jsf?id=1", Tools.generateLink("www.tenderdesk.cz/", "detail.jsf", parameters));
	}

	@Test
	public void shouldGetEnumAsText() {
		Assert.assertEquals("Chyba", Tools.getEnumAsText(LevelRecord.ERROR));
	}

	@Test
	public void shouldGetText() {
		Assert.assertEquals("Vytvořit", Tools.getText("button.create"));
	}

	@Test
	public void shouldGetKeyStringValue() {
		Assert.assertEquals("levelrecord.new", Tools.getKeyStringValue("levelRecord", "new"));
	}

	@Test
	public void shouldProvideDateAsText() {
		Assert.assertEquals("22.02.2016", Tools.provideDateAsText(prepareDate(), DateAndTimeFormat.FORMAT_OF_DATE));
	}

	@Test
	public void shouldProvideDateWithTime() {
		Assert.assertEquals("22.02.2016 00:00:00", Tools.provideDateWithTime(prepareDate()));
	}

	@Test
	public void shouldGetPathToServerFolder() {
		Assert.assertEquals(Tools.getPathToServerFolder("test.pdf"), Tools.getPathToServerFolder("test.pdf"));
	}

	@Test
	public void shouldGetFilePathSeparator() {
		Assert.assertNotNull(Tools.getFilePathSeparator());
	}

	@Test
	public void shouldIsEnum() {
		Assert.assertTrue(Tools.isEnum(LevelRecord.class));
	}

	@Test
	public void shouldIsNumver() {
		Assert.assertTrue(Tools.isNumber(Long.class));
	}

	private Date prepareDate() {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh.mm.ss");
		String dateInString = "2016-02-22 12.00.00";
		try {
			return sdf.parse(dateInString);
		} catch (ParseException e) {
			e.printStackTrace();
		}

		return null;
	}
}
