package cz.tendersystems.helpdesk.general.validation;

import java.text.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationPasswordTest {

	@Test
	public void shouldReturnFalseWhileUserIsNull() {
		Assert.assertFalse(ValidationPassword.isValidKey(null));

		try {
			ValidationPassword.validatePassword("", null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.keyIsInvalid", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhilePasswordHasWhiteCharacters() {
		try {
			ValidationPassword.validatePassword("  ", prepareUser());
		} catch (ValidationException e) {
			Assert.assertEquals("error.passwordContainsSpaces", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhilePasswordHasSevenCharacter() {
		try {
			ValidationPassword.validatePassword("passsss", prepareUser());
		} catch (ValidationException e) {
			Assert.assertEquals("error.passwordContainsAFewCharacters", e.getMessage());
		}
	}

	@Test
	public void shouldFailToPasswordMatchesRegex() {
		try {
			ValidationPassword.validatePassword("passs1234", prepareUser());
		} catch (ValidationException e) {
			Assert.assertEquals("error.passwordDoesNotMatchFormat", e.getMessage());
		}
	}

	@Test
	public void shouldValidatePassword() {
		try {
			ValidationPassword.validatePassword("pass1234A@", prepareUser());
		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}

	@Test
	public void shouldReturnTrueWhileKeyIsValid() {
		Assert.assertTrue(ValidationPassword.isValidKey(prepareUser()));
	}

	private UserSolver prepareUser() {
		UserSolver user = new UserSolver();
		ChangePassword changePassword = new ChangePassword();
		try {
			changePassword.setExpirationDate(new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").parse("2020-04-07 00:00:00"));
		} catch (ParseException e) {
			e.printStackTrace();
		}
		user.setChangePassword(changePassword);

		return user;
	}
}
