package cz.tendersystems.helpdesk.general.validation;

import org.junit.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationRequestTest {

	@Test
	public void shouldFailToWhileRequestHasNotSlaSchema() {
		try {
			ValidationRequest.validateRequest(new RequestData(), null, null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.slaSchemaMustNotNull", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileSlaSchemaIsNotActive() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(false);

		try {
			ValidationRequest.validateRequest(request, null, null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.slaSchemaMustActive", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileCompanyNameIsNull() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(true);
		request.setAuthor(new AuthorData());
		request.getAuthor().setAnonymous(new PersonData());

		try {
			ValidationRequest.validateRequest(request, null, null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.badAnonymousAuthor", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileLoggedUserIsNull() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(true);
		request.setAuthor(new AuthorData());
		request.getAuthor().setAnonymous(new PersonData());
		request.getAuthor().setCompanyName("test");
		request.getAuthor().setLoggedUser(new UserOutsourcerData());

		try {
			ValidationRequest.validateRequest(request, null, null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.badLoggedUser", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileCommentIsNull() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(true);
		request.setAuthor(new AuthorData());

		try {
			ValidationRequest.validateRequest(request, null, null);
		} catch (ValidationException e) {
			Assert.assertEquals("error.commentMustNotNull", e.getMessage());
		}
	}

	@Test
	public void shouldFailToRequestStatusIsSolved() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(true);
		request.setAuthor(new AuthorData());

		try {
			ValidationRequest.validateRequest(request, new CommentData(), RequestStatus.SOLVED);
		} catch (ValidationException e) {
			Assert.assertEquals("error.requesStatusCanNotBeSolved", e.getMessage());
		}
	}

	@Test
	public void shouldValidateRequest() {
		RequestData request = new RequestData();

		request.setSlaSchema(new SlaSchemaData());
		request.getSlaSchema().setActive(true);
		request.setAuthor(new AuthorData());

		try {
			ValidationRequest.validateRequest(request, new CommentData(), RequestStatus.INHERITED);
		} catch (ValidationException e) {
			e.printStackTrace();
		}
	}
}
