package cz.tendersystems.helpdesk.general.validation;

import java.text.*;
import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationSlaSchemaTest {

	@Test
	public void shouldFailToWhileAssignmentHasNotProduct() {
		AssignmentData data = new AssignmentData();

		try {
			ValidationSlaSchema.validationSlaSchema(data, new ArrayList<>());
		} catch (ValidationException e) {
			Assert.assertEquals("errors.slaSchemaMustHaveProduct", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileTimesActivityCollectionsIsEmpty() {
		AssignmentData data = new AssignmentData();
		data.setProduct(new ProductData());
		data.getProduct().setId(1l);

		try {
			ValidationSlaSchema.validationSlaSchema(data, new ArrayList<>());
		} catch (ValidationException e) {
			Assert.assertEquals("error.slaSchemaMustHaveAtLeastOneTimeActivity", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileTimesActivityCollectionHasNineObject() {
		AssignmentData data = new AssignmentData();
		data.setProduct(new ProductData());
		data.getProduct().setId(1l);

		try {
			ValidationSlaSchema.validationSlaSchema(data, Arrays.asList(new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData(), new TimeActivityData()));
		} catch (ValidationException e) {
			Assert.assertEquals("error.slaSchemaMustHaveMaxEightTimesActivity", e.getMessage());
		}
	}

	@Test
	public void shouldFailWhileStartTimeAfterEndTime() {
		AssignmentData data = new AssignmentData();
		data.setProduct(new ProductData());
		data.getProduct().setId(1l);

		TimeActivityData timeActivity = new TimeActivityData();
		timeActivity.setDayName(DayName.MONDAY);

		try {
			timeActivity.setStartTime(new SimpleDateFormat("hh:mm:ss").parse("13:00:00"));
			timeActivity.setEndTime(new SimpleDateFormat("hh:mm:ss").parse("08:00:00"));
			ValidationSlaSchema.validationSlaSchema(data, Arrays.asList(timeActivity));
		} catch (ParseException e) {
			e.printStackTrace();
		} catch (ValidationException e) {
			Assert.assertEquals("error.startTimeCanNotBeAfterEndTime", e.getMessage());
		}
	}

	@Test
	public void shouldExecutingAssignment() {
		AssignmentData data = new AssignmentData();
		data.setProduct(new ProductData());
		data.getProduct().setId(1l);

		TimeActivityData timeActivity = new TimeActivityData();
		timeActivity.setDayName(DayName.MONDAY);

		try {
			timeActivity.setStartTime(new SimpleDateFormat("hh:mm:ss").parse("08:00:00"));
			timeActivity.setEndTime(new SimpleDateFormat("hh:mm:ss").parse("13:00:00"));
			ValidationSlaSchema.validationSlaSchema(data, Arrays.asList(timeActivity));
		} catch (ParseException e) {
			e.printStackTrace();
		} catch (ValidationException e) {
			Assert.assertNull(e);
		}
	}

}
