package cz.tendersystems.helpdesk.general.validation;

import java.util.*;
import org.junit.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;

public class ValidationUserTest {

	@Test
	public void shouldFailToWhileUserSolverHasRoleTypeOutsourcer() {
		try {
			UserSolverData user = new UserSolverData();
			user.setRoleType(RoleType.OUTSOURCER);

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userSolverCanNotBeRoleTypeOutsourcer", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileUserOutsourcerHasRoleTypeAdministrator() {
		try {
			UserOutsourcerData user = new UserOutsourcerData();
			user.setRoleType(RoleType.ADMINISTRATOR);

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userOutsourcerMustHasRoleTypeOutsourcer", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileUserAdministratorHasProdcut() {
		try {
			UserSolverData user = new UserSolverData();
			user.setRoleType(RoleType.ADMINISTRATOR);
			user.setProducts(Arrays.asList(new ProductData()));

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userRoleAdministratorHasNotProducts", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileUserSolverHasNotProduct() {
		try {
			UserSolverData user = new UserSolverData();
			user.setRoleType(RoleType.SOLVER);

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userMustBeProduct", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileUserOutsourcerHasNotOutsourcer() {
		try {
			UserOutsourcerData user = new UserOutsourcerData();
			user.setRoleType(RoleType.OUTSOURCER);

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.userOutsourcerMustBeOutsourcer", e.getMessage());
		}
	}

	@Test
	public void shouldFailToWhileOutsourcerIsNotActive() {
		try {
			UserOutsourcerData user = new UserOutsourcerData();
			user.setRoleType(RoleType.OUTSOURCER);
			user.setOutsourcer(new OutsourcerData());

			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertEquals("error.selectedOutsourcerMustBeActive", e.getMessage());
		}
	}

	@Test
	public void shoudlValidateUserSolver() {
		UserSolverData user = new UserSolverData();
		user.setRoleType(RoleType.ADMINISTRATOR);

		try {
			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertNotNull(e.getMessage());
		}
	}

	@Test
	public void shoudlValidateUserOutsourcer() {
		UserOutsourcerData user = new UserOutsourcerData();
		user.setRoleType(RoleType.OUTSOURCER);
		user.setOutsourcer(new OutsourcerData());
		user.getOutsourcer().setActive(true);

		try {
			ValidationUser.validateUser(user);
		} catch (ValidationException e) {
			Assert.assertNotNull(e.getMessage());
		}
	}
}
