package cz.tendersystems.helpdesk.service.general.mail;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.service.configuration.*;

@RunWith(Arquillian.class)
public class MailTemplateServiceTest extends AbstractEJBTest {

	@EJB
	MailTemplateService mailTemplateService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(MailTemplateService.class, EmbedTemplate.class, ConfigurationService.class, VersionType.class);
		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");
		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	public void shouldUpdateTemplates() {
		mailTemplateService.update();

		Assert.assertNotNull(mailTemplateService.getTemplates());
	}

	@Test
	public void shouldPrepareTemplateForChangePassword() {
		mailTemplateService.update();

		String sablona = mailTemplateService.loadTemplate("linkForChangePassword");

		Assert.assertNotNull(sablona);
	}

	@Test
	public void shouldPrepareTemplateForFirstConfigurationPassword() {
		mailTemplateService.update();

		String sablona = mailTemplateService.loadTemplate("linkForFirstConfigurationPassword");

		Assert.assertNotNull(sablona);
	}

	@Test(expected = HelpdeskException.class)
	public void shouldFailToWhileNotExistTemplate() {
		mailTemplateService.update();

		try {
			mailTemplateService.loadTemplate("empty_template");
		} catch (HelpdeskException e) {
			Assert.assertEquals("Sablona emailu s nazvem empty_template neexistuje", e.getMessage());

			throw e;
		}
	}

	@Test
	public void shouldPrepareEmbedTemplate() {
		mailTemplateService.update();

		Assert.assertEquals("linkForChangePassword", mailTemplateService.addEmbedTemplate(EmbedTemplate.generalSalutationTemplate(VersionType.DEVELOPMENT.toString()), "linkForChangePassword"));
	}
}
