package cz.tendersystems.helpdesk.service.organizationalstructure;

import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;

@RunWith(Arquillian.class)
public class TestOrganizationalStructureServiceTest extends AbstractEJBTest {

	@EJB
	TestOrganizationalStructureService testOrganizationalStructureService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClass(TestOrganizationalStructureService.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	public void schouldCreateOrganizationalStructure() {
		testOrganizationalStructureService.createOrganizationalStrucure();
		;

		shouldCreateSolvers();
		shouldCreateUserOutsourcers();
		shouldCreateProducts();
		shouldCreateOutsourcers();
	}

	@SuppressWarnings("unchecked")
	private void shouldCreateSolvers() {
		List<UserSolver> solvers = em.createQuery("SELECT u FROM UserSolver u LEFT JOIN FETCH u.products").getResultList();

		Assert.assertEquals(solvers.size(), 5);

		for (UserSolver u : solvers) {
			Assert.assertTrue(u instanceof UserSolver);
			conditionsForUsers(u);

			if (!u.getRoleType().equals(RoleType.ADMINISTRATOR)) {
				Assert.assertNotNull(u.getProducts());
			} else {
				Assert.assertTrue(u.getProducts().isEmpty());
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void shouldCreateUserOutsourcers() {
		List<UserOutsourcer> outsourcers = em.createQuery("SELECT u FROM UserOutsourcer u LEFT JOIN FETCH u.outsourcer").getResultList();

		Assert.assertEquals(outsourcers.size(), 2);

		for (UserOutsourcer u : outsourcers) {
			Assert.assertTrue(u instanceof UserOutsourcer);
			conditionsForUsers(u);
			Assert.assertNotNull(u.getOutsourcer());
		}
	}

	private void conditionsForUsers(SystemUser uzivatel) {
		Assert.assertTrue(uzivatel.getLoginName().length() >= 5 && uzivatel.getLoginName().length() <= 50);
		Assert.assertTrue(uzivatel.getPassword().length() > 0 && uzivatel.getPassword().length() <= 40);
		Assert.assertEquals(uzivatel.getUserAccountStatus(), UserAccountStatus.ENABLED);
		Assert.assertNotNull(uzivatel.getCreatedDateTime());
		Assert.assertNotNull(uzivatel.getPerson());

		Assert.assertTrue(uzivatel.getPerson().getName().length() > 0 && uzivatel.getPerson().getName().length() <= 40);
		Assert.assertTrue(uzivatel.getPerson().getSurname().length() > 0 && uzivatel.getPerson().getSurname().length() <= 40);
		Assert.assertNotNull(uzivatel.getPerson().getMail());
	}

	@SuppressWarnings("unchecked")
	private void shouldCreateProducts() {
		List<Product> products = em.createQuery("SELECT p FROM Product p").getResultList();

		Assert.assertEquals(products.size(), 2);

		for (Product p : products) {
			Assert.assertTrue(p.getActive());
			Assert.assertNotNull(p.getId());
			Assert.assertNotNull(p.getName());
			Assert.assertTrue(p.getName().length() >= 1 && p.getName().length() <= 50);
		}
	}

	@SuppressWarnings("unchecked")
	private void shouldCreateOutsourcers() {
		List<Outsourcer> outsourcers = em.createQuery("SELECT o FROM Outsourcer o").getResultList();

		Assert.assertEquals(outsourcers.size(), 2);

		for (Outsourcer z : outsourcers) {
			Assert.assertTrue(z.getActive());
			Assert.assertNotNull(z.getId());
			Assert.assertNotNull(z.getCompanyName());
			Assert.assertNotNull(z.getCreatedDateTime());
			Assert.assertTrue(z.getCompanyName().length() >= 1 && z.getCompanyName().length() <= 50);
		}
	}
}
