package cz.tendersystems.helpdesk.service.organizationalstructure.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;

@RunWith(Arquillian.class)

public class OutsourcerListServiceTest extends AbstractEJBTest {

	@EJB
	OutsourcerListService outsourcerListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(OutsourcerListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, OutsourcerFilter.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			outsourcerListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam zadavatelu", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindAllProducts() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnId() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();
		filter.setId(5l);

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnActive() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();
		filter.setActive(true);

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(5, response.getData().size());
		Assert.assertEquals(Long.valueOf("6"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();
		filter.setCompanyName("Gordion");

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnDate() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();
		filter.setByDate(prepareDate("2014-01-00 00.00.00"));
		filter.setToDate(prepareDate("2016-02-00 00.00.00"));

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/outsourcerListServiceTest/outsourcers.xml", "datasets/outsourcerListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/outsourcerListServiceTest/cleanup.sql")
	public void shouldFindWithCombinationFilter() {
		login(LOGIN_NAME_ADMINISTRATOR);

		OutsourcerFilter filter = new OutsourcerFilter();
		filter.setByDate(prepareDate("2012-01-00 00.00.00"));
		filter.setToDate(prepareDate("2020-02-00 00.00.00"));
		filter.setActive(true);

		Response<OutsourcerData> response = outsourcerListService.find(new RequestList<OutsourcerFilter>(filter, 1, 10));

		Assert.assertEquals(4, response.getData().size());
		Assert.assertEquals(Long.valueOf("5"), response.getRecordCounter());
	}
}
