package cz.tendersystems.helpdesk.service.organizationalstructure.sla;

import java.text.*;
import java.util.*;
import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.entity.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.SecurityException;

@RunWith(Arquillian.class)
public class TimeActivityServiceTest extends AbstractEJBTest {

	@EJB
	TimeActivityService timeActivityService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClass(TimeActivityService.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@UsingDataSet("datasets/timeActivityService/timeActivityWithSla.xml")
	@ApplyScriptBefore("datasets/timeActivityService/cleanup.sql")
	public void shouldLoadTimesActivityOfSlaSchema() {
		List<TimeActivityData> timesActivity = timeActivityService.loadTimesActivityOfSlaSchema(1l);

		Assert.assertNotNull(timesActivity);
		Assert.assertEquals(5, timesActivity.size());
	}

	@Test(expected = SecurityException.class)
	public void shouldFailToUpdateTimesActivityOnSecurityException() {
		login(SOLVER_LOGIN_NAME);

		try {
			timeActivityService.updateTimesActivity(new ArrayList<>(), new SlaSchema());
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni upravit doby aktivity", e.getMessage());
			throw e;
		}
	}

	@Test
	@UsingDataSet("datasets/timeActivityService/slaSchema.xml")
	@ShouldMatchDataSet("datasets/timeActivityService/timeActivitySla-create.xml")
	@ApplyScriptBefore("datasets/timeActivityService/cleanup.sql")
	public void shouldCreateTimesActivity() {
		String HQL = "SELECT s FROM SlaSchema s LEFT JOIN FETCH s.timesActivity WHERE s.id = :idSlaSchema";

		login(LOGIN_NAME_ADMINISTRATOR);
		timeActivityService.updateTimesActivity(Arrays.asList(prepareTimeActivity(DayName.MONDAY, "08:00:00", "16:00:00"), prepareTimeActivity(DayName.TUESDAY, "08:00:00", "17:00:00")), (SlaSchema) em.createQuery(HQL).setParameter("idSlaSchema", 1L).getSingleResult());

		checkApplicationLog("Upravení dob aktivit", "Upravení dob aktivit u sla schéma s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	@Test
	@UsingDataSet("datasets/timeActivityService/timeActivityWithSla.xml")
	@ShouldMatchDataSet("datasets/timeActivityService/timeActivitySla-edit.xml")
	@ApplyScriptBefore("datasets/timeActivityService/cleanup.sql")
	public void shouldAddedTimesActivity() {
		String HQL = "SELECT s FROM SlaSchema s LEFT JOIN FETCH s.timesActivity WHERE s.id = :idSlaSchema";

		login(LOGIN_NAME_ADMINISTRATOR);
		timeActivityService.updateTimesActivity(Arrays.asList(prepareTimeActivity(DayName.HOLIDAY, "08:00:00", "16:00:00")), (SlaSchema) em.createQuery(HQL).setParameter("idSlaSchema", 1L).getSingleResult());

		checkApplicationLog("Upravení dob aktivit", "Upravení dob aktivit u sla schéma s id 1", LOGIN_NAME_ADMINISTRATOR, null);
	}

	private TimeActivityData prepareTimeActivity(DayName dayName, String startTime, String endTime) {
		TimeActivityData data = new TimeActivityData();
		data.setDayName(dayName);

		try {
			data.setStartTime(new SimpleDateFormat("hh:mm:ss").parse(startTime));
			data.setEndTime(new SimpleDateFormat("hh:mm:ss").parse(endTime));

		} catch (ParseException e) {
			e.printStackTrace();
		}

		return data;
	}
}
