package cz.tendersystems.helpdesk.service.organizationalstructure.sla.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;

@RunWith(Arquillian.class)
public class ProductListServiceTest extends AbstractEJBTest {

	@EJB
	ProductListService productListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(ProductListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, ProductFilter.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			productListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam produktu", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/productListServiceTest/products.xml", "datasets/productListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFindAllProducts() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductFilter filter = new ProductFilter();

		Response<ProductData> response = productListService.find(new RequestList<ProductFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/productListServiceTest/products.xml", "datasets/productListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnId() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductFilter filter = new ProductFilter();
		filter.setId(1l);

		Response<ProductData> response = productListService.find(new RequestList<ProductFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/productListServiceTest/products.xml", "datasets/productListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnActive() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductFilter filter = new ProductFilter();
		filter.setActive(true);

		Response<ProductData> response = productListService.find(new RequestList<ProductFilter>(filter, 1, 10));

		Assert.assertEquals(7, response.getData().size());
		Assert.assertEquals(Long.valueOf("8"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/productListServiceTest/products.xml", "datasets/productListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductFilter filter = new ProductFilter();
		filter.setName("ten");

		Response<ProductData> response = productListService.find(new RequestList<ProductFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/productListServiceTest/products.xml", "datasets/productListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/productListServiceTest/cleanup.sql")
	public void shouldFindWithCombinatioFilter() {
		login(LOGIN_NAME_ADMINISTRATOR);

		ProductFilter filter = new ProductFilter();
		filter.setName("e");
		filter.setActive(true);

		Response<ProductData> response = productListService.find(new RequestList<ProductFilter>(filter, 1, 10));

		Assert.assertEquals(6, response.getData().size());
		Assert.assertEquals(Long.valueOf("7"), response.getRecordCounter());
	}
}
