package cz.tendersystems.helpdesk.service.organizationalstructure.sla.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;

@RunWith(Arquillian.class)
public class SlaSchemaServiceTest extends AbstractEJBTest {

	@EJB
	SlaSchemaListService slaSchemaListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(SlaSchemaListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, SlaSchemaFilter.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			slaSchemaListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam sla schema", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindAllSlaSchemas() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnId() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();
		filter.setId(4l);

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnActive() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();
		filter.setActive(false);

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();
		filter.setName("1");

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();

		ProductData product = new ProductData();
		product.setId(1l);

		filter.setProduct(product);

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(4, response.getData().size());
		Assert.assertEquals(Long.valueOf("5"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/slaSchemaListServiceTest/slaSchemas.xml", "datasets/slaSchemaListServiceTest/user.xml" })
	@ApplyScriptBefore("datasets/slaSchemaListServiceTest/cleanup.sql")
	public void shouldFindWithCombinationFilter() {
		login(LOGIN_NAME_ADMINISTRATOR);

		SlaSchemaFilter filter = new SlaSchemaFilter();

		ProductData product = new ProductData();
		product.setId(1l);

		filter.setId(1l);
		filter.setActive(true);
		filter.setProduct(product);

		Response<SlaSchemaData> response = slaSchemaListService.find(new RequestList<SlaSchemaFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

}
