package cz.tendersystems.helpdesk.service.request.list;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.SecurityException;
import cz.tendersystems.helpdesk.service.list.*;
import cz.tendersystems.helpdesk.service.list.Filter;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@RunWith(Arquillian.class)
public class RequestListServiceTest extends AbstractEJBTest {

	@EJB
	RequestListService requestListService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();
		jar.addClasses(RequestListService.class, AbstractListService.class, RequestList.class, Filter.class, Response.class, RequestFilter.class);
		jar.addClasses(OutsourcerService.class, ProductService.class);

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFailToFindOnSecurityException() {
		try {
			requestListService.find(null);
		} catch (SecurityException e) {
			Assert.assertEquals("Nedostatecne opravneni nacist seznam pozadavku", e.getMessage());
		}
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindAdministratorRequests() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindResitelTaRequests() {
		login("resitelTa");

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindResitelTmRequests() {
		login("resitelTm");

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(5, response.getData().size());
		Assert.assertEquals(Long.valueOf("6"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindUzivatelTaRequests() {
		login("uzivatelTa");

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindUzivatelTmRequests() {
		login("uzivatelTm");

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(4, response.getData().size());
		Assert.assertEquals(Long.valueOf("5"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindPozorovatelTaRequests() {
		login("pozorovatelTa");

		RequestFilter filter = new RequestFilter();

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnId() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();
		filter.setId("7");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();
		filter.setName("tes");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(8, response.getData().size());
		Assert.assertEquals(Long.valueOf("9"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnRequestStatus() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();
		filter.setRequestStatus(RequestStatus.NEW);

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnProduct() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		ProductData product = new ProductData();
		product.setId(1l);

		filter.setProduct(product);

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnSlaSchema() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		SlaSchemaData slaSchema = new SlaSchemaData();
		slaSchema.setId(1l);
		slaSchema.setName("sla1");

		filter.setSlaSchema(slaSchema);

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnAuthorName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setName("zbyněk");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("0"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnSurname() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setAuthorSurname("rich");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("0"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnCpmpanyName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setAuthorCompanyName("tender");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(1, response.getData().size());
		Assert.assertEquals(Long.valueOf("2"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnSolver() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setSolver("res");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(7, response.getData().size());
		Assert.assertEquals(Long.valueOf("8"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnCreatedDateTime() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setByCreatedDateTime(prepareDate("2010-01-01 00.00.00"));
		filter.setToCreatedDateTime(prepareDate("2041-01-01 00:00:00"));

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(3, response.getData().size());
		Assert.assertEquals(Long.valueOf("4"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnReactionTime() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setByReactionTime(prepareDate("2000-01-01 00.00.00"));
		filter.setToReactionTime(prepareDate("2012-01-01 00:00:00"));

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnTimeToSolve() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setByTimeToSolve(prepareDate("2012-01-01 00.00.00"));
		filter.setToTimeToSolve(prepareDate("2012-12-30 00.00.00"));

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("0"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnDateOfClose() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setByDateOfClose(prepareDate("2008-01-01 00.00.00"));
		filter.setToDateOfClose(prepareDate("2016-01-20 00.00.00"));

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(0, response.getData().size());
		Assert.assertEquals(Long.valueOf("1"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnCombinationFilter1() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setByReactionTime(prepareDate("2000-01-01 00.00.00"));
		filter.setToReactionTime(prepareDate("2012-01-01 00:00:00"));
		filter.setId("7");

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(9, response.getData().size());
		Assert.assertEquals(Long.valueOf("10"), response.getRecordCounter());
	}

	@Test
	@UsingDataSet({ "datasets/requestListServiceTest/users.xml", "datasets/requestListServiceTest/requests.xml" })
	@ApplyScriptBefore("datasets/requestListServiceTest/cleanup.sql")
	public void shouldFindWithFilterOnCombinationFilter2() {
		login(LOGIN_NAME_ADMINISTRATOR);

		RequestFilter filter = new RequestFilter();

		filter.setName("tes");
		filter.setRequestStatus(RequestStatus.SOLVED);

		Response<RequestData> response = requestListService.find(new RequestList<RequestFilter>(filter, 1, 10));

		Assert.assertEquals(2, response.getData().size());
		Assert.assertEquals(Long.valueOf("3"), response.getRecordCounter());
	}
}
