package cz.tendersystems.helpdesk.service.security;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.entity.enums.*;

@RunWith(Arquillian.class)
public class BasicSecurityServiceTest extends AbstractEJBTest {

	@EJB
	BasicSecurityService basicSecurityService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	public void shouldLoadLoginName() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertEquals(LOGIN_NAME_ADMINISTRATOR, basicSecurityService.getLoginName());
	}

	@Test
	public void shouldReturnTrueWhileLoggedUser() {
		login(LOGIN_NAME_ADMINISTRATOR);
		Assert.assertTrue(basicSecurityService.isUserLogged());
	}

	@Test
	public void shouldReturnFalseWholeAnonymousUser() {
		Assert.assertFalse(basicSecurityService.isUserLogged());
	}

	@Test
	public void shouldReturnTrueWhileUserIsRoleTypeAdministrator() {
		login(LOGIN_NAME_ADMINISTRATOR);
		Assert.assertTrue(basicSecurityService.isUserAdministrator());
	}

	@Test
	public void shouldReturnFalseWhileUserIsNotRoleTypeAdministrator() {
		login(SOLVER_LOGIN_NAME);
		Assert.assertFalse(basicSecurityService.isUserAdministrator());
	}

	@Test
	public void shouldReturnTrueWhileUserIsRoleTypeResitel() {
		login(SOLVER_LOGIN_NAME);

		Assert.assertTrue(basicSecurityService.isRoleType(RoleType.SOLVER));
	}

	@Test
	public void shouldReturnTrueWhileUserIsNotRoleTypeResitel() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertFalse(basicSecurityService.isRoleType(RoleType.SOLVER));
	}
}
