package cz.tendersystems.helpdesk.service.security;

import javax.ejb.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.service.configuration.*;
import cz.tendersystems.helpdesk.service.general.mail.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@RunWith(Arquillian.class)
public class RequestSecurityServiceTest extends AbstractEJBTest {

	@EJB
	RequestSecurityService requestSecurityService;

	@Deployment
	public static Archive<?> createDeployment() {
		JavaArchive jar = prepareArchive();

		jar.addClasses(RequestSecurityService.class, UserService.class, SlaSchemaService.class, OutsourcerService.class, ProductService.class, VersionType.class);
		jar.addClasses(PasswordService.class, TimeActivityService.class, MailService.class, EmbedTemplate.class, MailTemplateService.class, ConfigurationService.class);
		jar.addAsResource("cz/tendersystems/helpdesk/configuration.properties");
		jar.addAsResource("cz/tendersystems/helpdesk/version.properties");

		System.out.println(jar.toString(true));

		return jar;
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityLoadRequestAnonymous.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityLoadRequestAnonymous() {
		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityLoadRequestAnonymous.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityLoadRequestAnonymous() {
		Assert.assertFalse(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityLoadRequestAdministrator.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityLoadRequestAdministrator() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityLoadRequestSolver.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityLoadRequestSolver() {
		login("resitelTa");

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityLoadRequestOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityLoadRequestOutsourcer() {
		login("uzivatelTa");

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityLoadRequestOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityLoadRequestOutsourcer() {
		login("uzivatelTa");

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityCreateCommentAnonymous.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityCreateCommentAnonymous() {
		Assert.assertTrue(requestSecurityService.authorityCreateComment("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityCreateCommentRequestIsArchived.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityCreateCommentRequestIsArchived() {
		Assert.assertFalse(requestSecurityService.authorityCreateComment("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityCreateCommentAdministrator.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityCreateCommentAdministrator() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityCreateCommentSolver.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturntrueWhileAuthorityCreateCommentSolver() {
		login("resitelTa");

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityCreateCommentOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityCreateCommenttOutsourcer() {
		login("uzivatelTa");

		Assert.assertTrue(requestSecurityService.authorityLoadRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0"));
	}

	@Test
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestAnonymous() {
		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("test", RequestStatus.ARCHIVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityUpdateRequestOutsourcer.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestOutsourcer() {
		login("uzivatelTa");

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("test", RequestStatus.ARCHIVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityUpdateRequestWatcher.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestWatcher() {
		login("pozorovatelTa");

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("test", RequestStatus.ARCHIVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityUpdateRequestEquelsRequestStatus.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestEquelsRequestStatus() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", RequestStatus.NEW));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityUpdateRequestRequestStatusIsArchived.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestRequestStatusIsArchived() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", RequestStatus.ARCHIVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityUpdateRequestAdministrator.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityUpdateRequestAdministrator() {
		login(LOGIN_NAME_ADMINISTRATOR);

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", RequestStatus.SOLVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnTrueWhileAuthorityUpdateRequestSolver.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnTrueWhileAuthorityUpdateRequestSolver() {
		login("resitelTa");

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", RequestStatus.SOLVED));
	}

	@Test
	@UsingDataSet("datasets/requestSecurityServiceTest/shouldReturnFalseWhileAuthorityUpdateRequestSolver.xml")
	@ApplyScriptBefore("datasets/requestSecurityServiceTest/cleanup.sql")
	public void shouldReturnFalseWhileAuthorityUpdateRequestSolver() {
		login("resitelTa");

		Assert.assertFalse(requestSecurityService.authorityUpdateRequest("1f59197e11c8618812f3eb6df7f06cdf6193f3b0", RequestStatus.SOLVED));
	}
}
