package cz.tendersystems.helpdesk;

import java.util.*;
import javax.annotation.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.export.*;
import cz.tendersystems.helpdesk.service.list.*;

public abstract class AbstractListAction<D, F extends Filter<?>> {

	public static final Integer NUMBER_FIRST_PAGE = 1;
	public static final Integer RECORD_COUNTER_ON_PAGE = 10;

	@Inject
	ExportListToPdfAction exportListToPdfAction;

	@Inject
	ExportListToXlsAction exportListToXlsAction;

	private F filter;
	private Integer page = NUMBER_FIRST_PAGE;
	private Integer recordCounterOnPage = RECORD_COUNTER_ON_PAGE;

	private List<D> data;
	private Long recordCounter;

	private Integer firstPage;
	private Integer previousPage;
	private Integer nextPage;
	private Integer lastPage;
	private List<Integer> pages = new ArrayList<Integer>();

	@PostConstruct
	public void initialize() {
		setFilter(prepareFilter());

		find();
	}

	public void find() {
		find(NUMBER_FIRST_PAGE);
	}

	public void find(Integer page) {
		RequestList<F> request = new RequestList<F>(getFilter(), (page - 1) * getRecordCounterOnPage(), getRecordCounterOnPage());
		Response<D> response = find(request);

		setData(response.getData());
		setRecordCounter(response.getRecordCounter());

		preparePages(page);
	}

	public void exportToPdf() {
		exportListToPdfAction.setColumnsWidth(prepareColumnsWidthExport());
		exportListToPdfAction.export(this.prepareRecordExport(), find(new RequestList<F>(getFilter(), 0, Integer.MAX_VALUE)));
	}

	public void exportToXls() {
		exportListToXlsAction.export(this.prepareRecordExport(), find(new RequestList<F>(getFilter(), 0, Integer.MAX_VALUE)));
	}

	protected float[] prepareColumnsWidthExport() {
		return new float[] {};
	}

	private void preparePages(Integer page) {
		setPage(page);
		setFirstPage(NUMBER_FIRST_PAGE);
		setPreviousPage(getPage() - 1);
		setNextPage(getPage() + 1);
		setLastPage((int) (getRecordCounter() / getRecordCounterOnPage() + (getRecordCounter() % getRecordCounterOnPage() > 0 ? 1 : 0)));

		prepareListPages();
	}

	private void prepareListPages() {
		pages.clear();

		int index = Math.min(Math.max(1, getLastPage() - 9), Math.max(1, getPage() - 4));
		for (int i = index; i < index + Math.min(10, getLastPage()); i++) {
			pages.add(i);
		}
	}

	public F getFilter() {
		return filter;
	}

	public void setFilter(F filter) {
		this.filter = filter;
	}

	public Integer getPage() {
		return page;
	}

	public void setPage(Integer page) {
		this.page = page;
	}

	public Integer getRecordCounterOnPage() {
		return recordCounterOnPage;
	}

	public void setRecordCounterOnPage(Integer recordCounterOnPage) {
		this.recordCounterOnPage = recordCounterOnPage;
	}

	public List<D> getData() {
		return data;
	}

	public void setData(List<D> data) {
		this.data = data;
	}

	public Long getRecordCounter() {
		return recordCounter;
	}

	public void setRecordCounter(Long recordCounter) {
		this.recordCounter = recordCounter;
	}

	public Integer getFirstPage() {
		return firstPage;
	}

	public void setFirstPage(Integer firstPage) {
		this.firstPage = firstPage;
	}

	public Integer getPreviousPage() {
		return previousPage;
	}

	public void setPreviousPage(Integer previousPage) {
		this.previousPage = previousPage;
	}

	public Integer getNextPage() {
		return nextPage;
	}

	public void setNextPage(Integer nextPage) {
		this.nextPage = nextPage;
	}

	public Integer getLastPage() {
		return lastPage;
	}

	public void setLastPage(Integer lastPage) {
		this.lastPage = lastPage;
	}

	public List<Integer> getPages() {
		return pages;
	}

	public void setPages(List<Integer> pages) {
		this.pages = pages;
	}

	protected abstract F prepareFilter();

	protected abstract Response<D> find(RequestList<F> response);

	protected abstract ReparationOfRecordExport<D> prepareRecordExport();
}
