package cz.tendersystems.helpdesk.export;

import java.awt.*;
import java.io.*;
import javax.enterprise.context.*;
import javax.inject.*;
import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.pdf.*;
import cz.tendersystems.helpdesk.service.list.*;

@Named
@RequestScoped
public class ExportListToPdfAction extends AbstractExportListAction {

	private Font font;
	private Color tableColor;
	private float[] columnsWidth;

	public ExportListToPdfAction() throws Exception {
		font = new Font(BaseFont.createFont(BaseFont.TIMES_ROMAN, BaseFont.CP1250, BaseFont.EMBEDDED), 7, Font.NORMAL);
		tableColor = Color.LIGHT_GRAY;
	}

	public void setColumnsWidth(float[] columnsWidth) {
		this.columnsWidth = columnsWidth;
	}

	@Override
	protected String prepareContentType() {
		return "application/pdf";
	}

	@Override
	protected String prepareSuffix() {
		return "pdf";
	}

	@Override
	protected <D, F extends Filter<?>> void prepareDocument(ReparationOfRecordExport<D> action, Response<D> response, OutputStream output) throws Exception {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Document document = new Document();
		PdfWriter.getInstance(document, baos);

		document.setMargins(4, 4, 30, 30);

		document.open();
		document.add(prepareData(action, response));
		document.close();

		baos.flush();
		baos.close();
		output.write(baos.toByteArray());
	}

	private <D, F extends Filter<?>> PdfPTable prepareData(ReparationOfRecordExport<D> action, Response<D> response) throws DocumentException {
		PdfPTable table = new PdfPTable(action.prepareHeader().size());

		table.setWidthPercentage(columnsWidth, PageSize.A2);
		table.getDefaultCell().setPaddingBottom(5);
		table.getDefaultCell().setBorderColor(tableColor);
		table.setSplitLate(false);

		insertHeader(table, action);
		insertData(table, action, response);

		return table;
	}

	private <D, F extends Filter<?>> void insertHeader(PdfPTable table, ReparationOfRecordExport<D> action) {
		for (String key : action.prepareHeader()) {
			PdfPCell cell = new PdfPCell(new Phrase(localize(key), font));

			cell.setBackgroundColor(tableColor);
			cell.setBorderColor(Color.WHITE);
			cell.setPadding(5);

			table.addCell(cell);
		}
	}

	private <D, F extends Filter<?>> void insertData(PdfPTable table, ReparationOfRecordExport<D> action, Response<D> response) {
		for (D record : response.getData()) {
			for (Object atribute : action.prepareRecord(record)) {
				table.addCell(new Phrase(reparationStringValueConverter.prepareStringValue(atribute), font));
			}
		}
	}
}