package cz.tendersystems.helpdesk.export;

import java.io.*;
import java.util.*;
import javax.enterprise.context.*;
import javax.inject.*;
import org.apache.poi.hssf.usermodel.*;
import cz.tendersystems.helpdesk.service.list.*;

@Named
@RequestScoped
public class ExportListToXlsAction extends AbstractExportListAction {

	@Override
	protected String prepareContentType() {
		return "application/vnd.ms-excel";
	}

	@Override
	protected String prepareSuffix() {
		return "xls";
	}

	@Override
	protected <D, F extends Filter<?>> void prepareDocument(ReparationOfRecordExport<D> action, Response<D> response, OutputStream output) throws Exception {
		HSSFWorkbook sheet = new HSSFWorkbook();
		HSSFSheet list = sheet.createSheet(prepareFileName());

		insertHeader(sheet, list, action);
		insertData(sheet, list, action, response);

		sheet.write(output);
	}

	@SuppressWarnings("deprecation")
	private <D, F extends Filter<?>> void insertHeader(HSSFWorkbook sheet, HSSFSheet list, ReparationOfRecordExport<D> action) throws Exception {
		HSSFRow row = list.createRow(0);

		Iterator<String> iterator = action.prepareHeader().iterator();
		int column = 0;

		while (iterator.hasNext()) {
			row.createCell(column).setCellValue(localize(iterator.next()));
			column++;
		}
	}

	@SuppressWarnings("deprecation")
	private <D, F extends Filter<?>> void insertData(HSSFWorkbook sheet, HSSFSheet list, ReparationOfRecordExport<D> action, Response<D> response) throws Exception {
		for (int i = 0; i < response.getRecordCounter(); i++) {
			HSSFRow row = list.createRow(i + 1);

			Iterator<Object> iterator = action.prepareRecord(response.getData().get(i)).iterator();
			int column = 0;

			while (iterator.hasNext()) {
				row.createCell(column).setCellValue(reparationStringValueConverter.prepareStringValue(iterator.next()));
				column++;
			}
		}
	}
}