package cz.tendersystems.helpdesk.general;

import java.io.*;
import java.util.*;
import javax.enterprise.context.*;
import javax.faces.context.*;
import javax.faces.view.facelets.*;
import javax.inject.*;
import javax.servlet.http.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.servlet.*;

@Named
@SessionScoped
public class BackNavigationAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = -2977480011727495969L;

	private static final String backPage = "backPage";

	public boolean showButtonBack() {
		String uiParameter = getPageUiParameter(backPage);
		return uiParameter != null && isGoBack();
	}

	public String prepareBackPageOrRedirect() throws IOException {
		String uiParameter = getPageUiParameter(backPage);

		removePageFromNavigation();

		return uiParameter;
	}

	private boolean isGoBack() {
		Deque<String> navigation = prepareNavigation();

		return navigation != null && !navigation.isEmpty() && !navigation.peekLast().equals(BackNavigationFilter.NO_ELEMENT);
	}

	private String getPageUiParameter(String parameterName) {
		FaceletContext faceletContext = (FaceletContext) FacesContext.getCurrentInstance().getAttributes().get(FaceletContext.FACELET_CONTEXT_KEY);

		return faceletContext.getAttribute(parameterName) != null ? faceletContext.getAttribute(parameterName).toString() : null;
	}

	private void removePageFromNavigation() {
		Deque<String> navigation = prepareNavigation();

		if (!navigation.isEmpty()) {
			navigation.removeLast();
			navigation.addLast(BackNavigationFilter.NO_ELEMENT);
		}
	}

	@SuppressWarnings("unchecked")
	private Deque<String> prepareNavigation() {
		HttpSession session = (HttpSession) FacesContext.getCurrentInstance().getExternalContext().getSession(true);
		Deque<String> navigation = (Deque<String>) session.getAttribute(BackNavigationFilter.NAVIGATION_KEY);

		return navigation;
	}
}
