package cz.tendersystems.helpdesk.general.converters;

import java.text.*;
import java.util.*;
import javax.enterprise.context.*;
import javax.faces.component.*;
import javax.faces.context.*;
import javax.faces.convert.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.general.*;

@Named
@RequestScoped
public class DateConverter implements Converter {

	private static final String FORMAT_TYPE = "type";
	private static final String FORMAT_OF_DATE = "date";
	private static final String FORMAT_OF_DATE_AND_TIME = "dateAndTime";
	private static final String FORMAT_OF_TIME = "time";

	@Override
	public Object getAsObject(FacesContext context, UIComponent component, String stringValue) {
		try {
			return new SimpleDateFormat(getFormatData(getFormatType(component))).parse(stringValue != null ? stringValue : "");
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public String getAsString(FacesContext context, UIComponent component, Object object) {
		if (object instanceof Date) {
			String format = getFormatType(component);
			Date date = (Date) object;

			return new SimpleDateFormat(getFormatData(format), Tools.LOCALIZATION_OF_FORMATTER).format(date);
		}

		throw new IllegalArgumentException(MessageFormat.format("Object {0} is not a date", object));
	}

	public String provideDateAsText(Date date, DateAndTimeFormat formatType) {
		return formatType.provideDateAsText(date);
	}

	public String provideDateFormatAsText(Date date, String formatType) {
		return new SimpleDateFormat(getFormatData(formatType)).format(date);
	}

	public String getLocalization() {
		return Tools.LOCALIZATION;
	}

	public String getFormatOfDate() {
		return getFormatData(FORMAT_OF_DATE);
	}

	public String getFormatOfDateAndTime() {
		return getFormatData(FORMAT_OF_DATE_AND_TIME);
	}

	public String getFormatOfTime() {
		return getFormatData(FORMAT_OF_TIME);
	}

	public String getFormatData(String formatType) {
		switch (formatType) {
			case FORMAT_OF_DATE:
				return DateAndTimeFormat.FORMAT_OF_DATE.getFormat();
			case FORMAT_OF_TIME:
				return DateAndTimeFormat.FORMAT_OF_TIME.getFormat();
			case FORMAT_OF_DATE_AND_TIME:
				return DateAndTimeFormat.FORMAT_OF_DATE_AND_TIME.getFormat();
			default:
				return DateAndTimeFormat.FORMAT_OF_DATE.getFormat();
		}
	}

	private String getFormatType(UIComponent component) {
		return component.getAttributes().get(FORMAT_TYPE) == null ? FORMAT_OF_DATE : component.getAttributes().get(FORMAT_TYPE).toString();
	}
}