package cz.tendersystems.helpdesk.general.converters;

import java.text.*;
import java.util.regex.*;
import javax.enterprise.context.*;
import javax.faces.application.*;
import javax.faces.component.*;
import javax.faces.context.*;
import javax.faces.convert.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.general.*;

@Named
@RequestScoped
public class NumberConverter implements Converter {

	private static final String numberType = "type";
	private static final String emptySymbols = "\\s|\\xA0";
	private static final Pattern formatOfWholeNumber = Pattern.compile("\\d+");
	private static final Pattern formatOfDecimalNumber = Pattern.compile("\\d+([,]\\d{0,4})?");

	@Override
	public Object getAsObject(FacesContext context, UIComponent component, String stringValue) {
		if (stringValue == null) {
			return null;
		}

		Class<?> numberType = getNumberType(component, null);
		String modifiedString = stringValue.replaceAll(emptySymbols, "");

		try {
			if (formatOfWholeNumber.matcher(modifiedString).matches() || formatOfDecimalNumber.matcher(modifiedString).matches()) {
				if (Integer.class.equals(numberType)) {
					return ((Long) Tools.FORMATTER_OF_WHOLE_NUMBERS.parse(modifiedString)).intValue();
				} else if (Long.class.equals(numberType)) {
					return Tools.FORMATTER_OF_WHOLE_NUMBERS.parse(modifiedString);
				}
			}

			throw new UnsupportedOperationException();
		} catch (Exception e) {
			FacesContext.getCurrentInstance().addMessage(component.getClientId(), new FacesMessage(FacesMessage.SEVERITY_ERROR, null, MessageFormat.format("Hodnota {0} neodpovida formatu", stringValue)));
			throw new ConverterException();
		}
	}

	@Override
	public String getAsString(FacesContext context, UIComponent component, Object object) {
		Class<?> numberType = getNumberType(component, object);

		if (object == null) {
			return null;
		} else if (Integer.class.equals(numberType) || Long.class.equals(numberType)) {
			return Tools.FORMATTER_OF_WHOLE_NUMBERS.format(object);
		} else {
			throw new UnsupportedOperationException();
		}
	}

	private Class<?> getNumberType(UIComponent component, Object object) {
		return component == null || component.getAttributes().get(numberType) == null ? object == null ? null : object.getClass() : (Class<?>) component.getAttributes().get(numberType);
	}
}