package cz.tendersystems.helpdesk.general.jsf.messages;

import java.util.*;
import javax.faces.application.*;
import javax.faces.context.*;
import javax.faces.event.*;
import org.jboss.logging.*;

public class FacesMessagesPhaseListener implements PhaseListener {

	private static final long serialVersionUID = 2214926505872648430L;

	private static final String flashMessagesSupport = "FLASH_MESSAGES_SUPPORT";
	private static final Logger LOG = Logger.getLogger(FacesMessagesPhaseListener.class);

	@Override
	public void afterPhase(PhaseEvent event) {
		if (!PhaseId.RENDER_RESPONSE.equals(event.getPhaseId())) {
			createFacesMessages(event.getFacesContext());
		}
	}

	@Override
	public void beforePhase(PhaseEvent event) {
		FacesContext context = event.getFacesContext();
		createFacesMessages(context);

		if (PhaseId.RENDER_RESPONSE.equals(event.getPhaseId()) && !context.getResponseComplete()) {
			loadFacesMessages(context);
		}
	}

	@Override
	public PhaseId getPhaseId() {
		return PhaseId.ANY_PHASE;
	}

	@SuppressWarnings("unchecked")
	private void createFacesMessages(FacesContext context) {
		if (context.getExternalContext().getSession(false) != null) {
			LOG.tracef("Ukladam faces messages");

			Map<String, Object> sessionMap = context.getExternalContext().getSessionMap();

			if (!sessionMap.containsKey(flashMessagesSupport)) {
				sessionMap.put(flashMessagesSupport, new ArrayList<FacesMessage>());
			}

			Iterator<FacesMessage> iterator = context.getMessages();
			List<FacesMessage> facesMessages = (List<FacesMessage>) sessionMap.get(flashMessagesSupport);

			while (iterator.hasNext()) {
				FacesMessage message = iterator.next();

				if (message instanceof GeneralFacesMessage) {
					facesMessages.add(message);
					iterator.remove();
				}
			}

			LOG.tracef("Faces messages ulozeny (celkem {0})", facesMessages.size());
		}
	}

	@SuppressWarnings("unchecked")
	private void loadFacesMessages(FacesContext context) {
		if (context.getExternalContext().getSession(false) != null) {
			LOG.tracef("Nacitam faces messages");

			List<FacesMessage> facesMessages = (List<FacesMessage>) context.getExternalContext().getSessionMap().remove(flashMessagesSupport);

			for (FacesMessage facesMessage : facesMessages) {
				context.addMessage(null, facesMessage);
			}

			LOG.tracef("Faces messages nacteny (celkem {0})", facesMessages.size());
		}
	}
}