package cz.tendersystems.helpdesk.general.jsf.resources;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.faces.application.*;

public abstract class AbstractResource extends ResourceWrapper {

	private static final ResourceBundle configuration = ResourceBundle.getBundle("cz.tendersystems.helpdesk.general.jsf.resources.configuration");
	private static final String configurationPath = "/cz/tendersystems/helpdesk/general/jsf/resources/configuration.properties";
	protected static final Properties properties = new Properties();

	static {
		try {
			InputStream input = AbstractResource.class.getResourceAsStream(configurationPath);
			properties.load(input);
			input.close();
		} catch (Throwable e) {
			throw new Error(e);
		}
	}

	protected String id;
	protected Resource wrapper;

	public AbstractResource(String id, Resource wrapper) {
		this.id = id;
		this.wrapper = wrapper;
	}

	@Override
	public Resource getWrapped() {
		return wrapper;
	}

	@Override
	public InputStream getInputStream() throws IOException {
		try {
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			copy(super.getInputStream(), output);

			return new ByteArrayInputStream(processData(provideDefaultLayoutCustomization(), output.toByteArray()));
		} catch (Exception e) {
			throw new Error(e);
		}
	}

	private static void copy(InputStream is, OutputStream os) throws IOException {
		byte[] buffer = new byte[1024];
		int read = 0;
		while ((read = is.read(buffer, 0, buffer.length)) != -1) {
			os.write(buffer, 0, read);
		}
	}

	private static LayoutCustomization provideDefaultLayoutCustomization() {
		LayoutCustomization layoutCustomization = new LayoutCustomization();

		layoutCustomization.setFirstMainColor(configuration.getString("@firstMainColor"));
		layoutCustomization.setFirstAdditionColor(configuration.getString("@firstAdditionColor"));

		layoutCustomization.setSecondMainColor(configuration.getString("@secondMainColor"));
		layoutCustomization.setSecondAdditionColor(configuration.getString("@secondAdditionColor"));

		layoutCustomization.setThirdMainColor(configuration.getString("@thirdMainColor"));
		layoutCustomization.setThirdAdditionColor(configuration.getString("@thirdAdditionColor"));

		layoutCustomization.setInputText(configuration.getString("@inputColor"));
		layoutCustomization.setBackgroundMessageColor(configuration.getString("@backgroundMessageColor"));

		return layoutCustomization;
	}

	protected abstract byte[] processData(LayoutCustomization prizpusobeniVzhledu, byte[] data);

	@Override
	public String getRequestPath() {
		return MessageFormat.format("{0}?ln=helpdesk.{1}", super.getRequestPath(), id);
	}
}