package cz.tendersystems.helpdesk.general.jsf.resources;

import java.text.*;
import java.util.regex.*;
import javax.faces.application.*;

public class CustomResourceHandler extends ResourceHandlerWrapper {

	private static final Pattern patternForLibrary = Pattern.compile("helpdesk[.](.*)");
	private static final Pattern patternForImage = Pattern.compile("image/.*");
	private static final Pattern patternForStyle = Pattern.compile("text/css");

	private ResourceHandler wrapper;

	public CustomResourceHandler(ResourceHandler wrapper) {
		this.wrapper = wrapper;
	}

	@Override
	public ResourceHandler getWrapped() {
		return wrapper;
	}

	@Override
	public Resource createResource(String resourceName, String libraryName) {
		if (libraryName != null) {
			Matcher matcher = patternForLibrary.matcher(libraryName);

			if (matcher.matches()) {
				Resource resource = wrapper.createResource(resourceName);

				if (resource != null && resource.getContentType() != null) {
					if (patternForImage.matcher(resource.getContentType()).matches()) {
						return new ImageResource(matcher.group(1), resource);
					} else if (patternForStyle.matcher(resource.getContentType()).matches()) {
						return new StyleResource(matcher.group(1), resource);
					} else {
						throw new UnsupportedOperationException(MessageFormat.format("Neznamy typ obsahu zdroje {0}", resourceName));
					}
				}
			}
		}

		return wrapper.createResource(resourceName, libraryName);
	}
}