package cz.tendersystems.helpdesk.organizationalstructure;

import java.io.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.faces.application.*;
import javax.faces.component.*;
import javax.faces.context.*;
import javax.faces.event.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.jsf.extensions.*;
import cz.tendersystems.helpdesk.general.jsf.parameter.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.security.*;

@Named
@ViewScoped
public class PasswordAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = 9084644213216078681L;
	private static final String index = "home";
	private static final String userDetailFromChangePassword = "userDetailFromChangePassword";

	@EJB
	PasswordService passwordService;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	UserService userService;

	@Inject
	@Parameter
	private String key;

	@Inject
	@Parameter
	private String loginName;

	private String newPassword;
	private String newPasswordConfirmation;
	private boolean validKey;
	private UserType userType;

	@PostConstruct
	public void initialize() {
		if (loginName != null) {
			userType = userService.loadUser(loginName).getRoleType().getUserType();
		}

		validKey = key != null && passwordService.checkTheValidityOfKey(key);
	}

	public String sendLinkForChangePassword() {
		try {
			passwordService.sendLinkForChangePassword(loginName);

			writeFacesMessage("information.successfulSendLinkForChangePassword", LogLevel.INFORMATION);

			return basicSecurityService.isUserLogged() ? userDetailFromChangePassword : index;
		} catch (ValidationException e) {
			writeFacesMessage(e, LogLevel.WARNING);
		} catch (HelpdeskException e) {
			writeFacesMessage(e, LogLevel.WARNING);
		}

		return "";
	}

	public String changePassword() {
		try {
			passwordService.changePassword(key, newPassword);

			writeFacesMessage("information.successfulChangePassword", LogLevel.INFORMATION);

			return index;
		} catch (ValidationException e) {
			writeFacesMessage(e, LogLevel.WARNING);

			return "";
		}
	}

	public void checkPassword(ComponentSystemEvent event) {
		FacesContext context = FacesContext.getCurrentInstance();
		UIComponent component = event.getComponent();

		UIInput newPasswordArray = (UIInput) component.findComponent("newPassword");
		UIInput newPasswordConfirmationArray = (UIInput) component.findComponent("newPasswordConfirmation");

		String newPassword = newPasswordArray.getLocalValue() == null ? "" : newPasswordArray.getLocalValue().toString();
		String newPasswordConfirmation = newPasswordConfirmationArray.getLocalValue() == null ? "" : newPasswordConfirmationArray.getLocalValue().toString();

		if (!(newPassword != null && newPassword.equals(newPasswordConfirmation))) {
			context.addMessage(newPasswordConfirmationArray.getClientId(), new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Tools.getText("error.newPasswordAndConfirmIsNotSame")));
			context.renderResponse();
		}
	}

	public String getLoginName() {
		return loginName;
	}

	public void setLoginName(String loginName) {
		this.loginName = loginName;
	}

	public String getNewPassword() {
		return newPassword;
	}

	public void setNewPassword(String newPassword) {
		this.newPassword = newPassword;
	}

	public String getNewPasswordConfirmation() {
		return newPasswordConfirmation;
	}

	public void setNewPasswordConfirmation(String newPasswordConfirmation) {
		this.newPasswordConfirmation = newPasswordConfirmation;
	}

	public boolean isValidKey() {
		return validKey;
	}

	public UserType getUserType() {
		return userType;
	}
}
