package cz.tendersystems.helpdesk.organizationalstructure;

import java.io.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.jsf.extensions.*;
import cz.tendersystems.helpdesk.general.jsf.parameter.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@Named
@ViewScoped
public class UserAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = -5322707156347481463L;

	@EJB
	UserService userService;

	@EJB
	ProductService productService;

	@EJB
	OutsourcerService outSourcerService;

	@Inject
	@Parameter
	private String loginName;

	@Inject
	@Parameter
	private String userType;

	private SystemUserData user;

	@PostConstruct
	public void initialize() {
		if (loginName == null) {
			user = userType.equals(UserType.OUTSOURCER.toString()) ? new UserOutsourcerData() : (userType.equals(UserType.SOLVER.toString()) ? new UserSolverData() : null);

			if (user == null) {
				throw new HelpdeskException("Instance uzivatele neexistuje");
			}

			return;
		}

		user = userService.loadUser(loginName);
	}

	public String createUser() {
		try {
			loginName = userService.createUser(user);
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.ERROR);

			return "";
		}

		writeFacesMessage("information.successfulUserCreation", LogLevel.INFORMATION);

		return "detailUser";
	}

	public String updateUser() {
		try {
			userService.updateUser(user);
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.ERROR);

			return "";
		}

		writeFacesMessage("information.successfulUserUpdate", LogLevel.INFORMATION);

		return "detailUser";
	}

	public void addProduct(ProductData product) {
		if (((UserSolverData) user).getProducts().contains(product)) {
			writeFacesMessage("warning.productHadAlreadyBeenAdded", LogLevel.WARNING);

			return;
		}

		((UserSolverData) user).getProducts().add(product);
	}

	public void addOutsourcer(OutsourcerData outsourcer) {
		if (((UserOutsourcerData) user).getOutsourcer() != null) {
			if (((UserOutsourcerData) user).getOutsourcer().equals(outsourcer)) {
				writeFacesMessage("warning.outsourcerHadAlreadyBeenAdded", LogLevel.WARNING);

				return;
			}
		}

		((UserOutsourcerData) user).setOutsourcer(outsourcer);
	}

	public void removeProduct(ProductData product) {
		((UserSolverData) user).getProducts().remove(product);
	}

	public void enabledUser() {
		updateUserAccountStatus(UserAccountStatus.ENABLED);
	}

	public void forbiddenUser() {
		updateUserAccountStatus(UserAccountStatus.FORBIDDEN);
	}

	private void updateUserAccountStatus(UserAccountStatus status) {
		userService.changeUserAcountStatus(loginName, status);

		writeFacesMessage("information.successfulUserAccountStatusUpdate", LogLevel.INFORMATION);
	}

	public String getLoginName() {
		return loginName;
	}

	public String getUserType() {
		return userType;
	}

	public SystemUserData getUser() {
		return user;
	}

	public void setUser(SystemUserData user) {
		this.user = user;
	}
}
