package cz.tendersystems.helpdesk.organizationalstructure.sla;

import java.io.*;
import java.util.*;
import javax.annotation.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.jsf.extensions.*;
import cz.tendersystems.helpdesk.general.jsf.parameter.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;

@Named
@ViewScoped
public class SlaSchemaAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = -1230237928011376495L;

	@Inject
	SlaSchemaService slaSchemaService;

	@Inject
	TimeActivityService timeActivityService;

	@Inject
	@Parameter
	private Long id;

	private AssignmentData assignment;
	private List<TimeActivityData> timesActivity = new ArrayList<>();
	private TimeActivityData timeActivity = new TimeActivityData();

	@PostConstruct
	public void initialize() {
		if (id == null) {
			assignment = new AssignmentData();

			return;
		}

		assignment = slaSchemaService.loadAssignmentSlaSchema(id);
		timesActivity = timeActivityService.loadTimesActivityOfSlaSchema(id);
	}

	public String createSlaSchema() {
		try {
			id = slaSchemaService.createSlaSchema(assignment, timesActivity);
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.WARNING);

			return "";
		}

		writeFacesMessage("information.successfulSlaSchemaCreation", LogLevel.INFORMATION);

		return "detailSlaSchema";
	}

	public String updateSlaSchema() {
		try {
			slaSchemaService.updateSlaSchema(assignment, timesActivity);
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.ERROR);

			return "";
		}

		writeFacesMessage("information.successfulSlaSchemaUpdate", LogLevel.INFORMATION);

		return "detailSlaSchema";
	}

	public void enabledSlaSchema() {
		updateActivityOfProduct(true);
	}

	public void forbiddenSlaSchema() {
		updateActivityOfProduct(false);
	}

	private void updateActivityOfProduct(boolean active) {
		slaSchemaService.updateActivityOfSlaSchema(assignment.getSlaSchema(), active);

		writeFacesMessage("information.successfulActivityOfSlaSchemaUpdate", LogLevel.INFORMATION);
	}

	public void addTimeActivity() {
		if (timesActivity.contains(timeActivity)) {
			writeFacesMessage("warning.timeActivityHadAlreadyBeenAdded", LogLevel.WARNING);

			return;
		}

		timesActivity.add(timeActivity);
		timeActivity = new TimeActivityData();
	}

	public void removeTimeActivity(TimeActivityData timeActivity) {
		timesActivity.remove(timeActivity);
	}

	public void addProduct(ProductData product) {
		if (assignment.getProduct() != null) {
			if (assignment.getProduct().equals(product)) {
				writeFacesMessage("warning.productHadAlreadyBeenAdded", LogLevel.WARNING);

				return;
			}
		}

		assignment.setProduct(product);
	}

	public void addOutsourcer(OutsourcerData outsourcer) {
		if (assignment.getOutsourcer() != null) {
			if (assignment.getOutsourcer().equals(outsourcer)) {
				writeFacesMessage("warning.outsourcerHadAlreadyBeenAdded", LogLevel.WARNING);

				return;
			}
		}

		assignment.setOutsourcer(outsourcer);
	}

	public void removeOutsourcer() {
		assignment.setOutsourcer(null);
	}

	public Long getId() {
		return id;
	}

	public AssignmentData getAssignment() {
		return assignment;
	}

	public void setAssignment(AssignmentData assignment) {
		this.assignment = assignment;
	}

	public List<TimeActivityData> getTimesActivity() {
		return timesActivity;
	}

	public void setTimesActivity(List<TimeActivityData> timesActivity) {
		this.timesActivity = timesActivity;
	}

	public TimeActivityData getTimeActivity() {
		return timeActivity;
	}

	public void setTimeActivity(TimeActivityData timeActivity) {
		this.timeActivity = timeActivity;
	}
}
