package cz.tendersystems.helpdesk.request;

import java.io.*;
import java.util.*;
import javax.annotation.*;
import javax.ejb.*;
import javax.inject.*;
import cz.tendersystems.helpdesk.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.embeddable.*;
import cz.tendersystems.helpdesk.data.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.data.request.*;
import cz.tendersystems.helpdesk.entity.enums.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.general.*;
import cz.tendersystems.helpdesk.general.jsf.extensions.*;
import cz.tendersystems.helpdesk.general.jsf.parameter.*;
import cz.tendersystems.helpdesk.service.general.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.*;
import cz.tendersystems.helpdesk.service.organizationalstructure.sla.*;
import cz.tendersystems.helpdesk.service.request.*;
import cz.tendersystems.helpdesk.service.security.*;

@Named
@ViewScoped
public class RequestAction extends AbstractAction implements Serializable {

	private static final long serialVersionUID = 5630557384886758398L;

	@Inject
	FileUploadAction fileUploadAction;

	@EJB
	BasicSecurityService basicSecurityService;

	@EJB
	UserService userService;

	@EJB
	ProductService productService;

	@EJB
	RequestService requestService;

	@EJB
	SlaSchemaService slaSchemaService;

	@EJB
	CommentService commentService;

	@EJB
	FileService fileService;

	@Inject
	@Parameter
	private String id;

	@Inject
	@Parameter
	private Long idProduct;

	private RequestData request;
	private List<CommentData> comments = new ArrayList<>();
	private CommentData comment = new CommentData();
	private List<FileData> files = new ArrayList<>();

	@PostConstruct
	public void initialize() {
		if (id == null) {
			request = new RequestData();

			prepareAuthor();

			return;
		}

		request = requestService.loadRequest(id);
		comments = commentService.loadCommentRequest(id);
		files = fileService.loadFilesRequest(id);
	}

	private void prepareAuthor() {
		if (basicSecurityService.isUserLogged() && basicSecurityService.isRoleType(RoleType.OUTSOURCER)) {
			request.getAuthor().setLoggedUser((UserOutsourcerData) userService.loadUser(basicSecurityService.getLoginName()));
		} else {
			request.getAuthor().setAnonymous(new PersonData());
		}

	}

	public String createRequest() {
		try {
			id = requestService.createRequest(request, comment, fileUploadAction.getFiles());

			writeFacesMessage("information.successfulRequestCreation", LogLevel.INFORMATION);
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.ERROR);

			return "";
		}

		return "detailRequest";
	}

	public String createComment() {
		commentService.createComment(request.getId(), comment, fileUploadAction.getFiles());

		writeFacesMessage("information.successfulCommentCreation", LogLevel.INFORMATION);

		return "detailRequest";
	}

	public String UpdateRequest(RequestStatus status) {
		try {
			requestService.updateRequest(request, comment, status, fileUploadAction.getFiles());

			writeFacesMessage("information.successfulUpdateRequest", LogLevel.INFORMATION);

			return "detailRequest";
		} catch (ValidationException e) {
			writeFacesMessage(e.getMessage(), LogLevel.ERROR);

			return "";
		}
	}

	public String getId() {
		return id;
	}

	public Long getIdProduct() {
		return idProduct;
	}

	public ProductData getProduct() {
		return idProduct == null ? slaSchemaService.loadAssignmentSlaSchema(request.getSlaSchema().getId()).getProduct() : productService.loadProduct(idProduct);
	}

	public RequestData getRequest() {
		return request;
	}

	public void setRequest(RequestData request) {
		this.request = request;
	}

	public CommentData getComment() {
		return comment;
	}

	public void setComment(CommentData comment) {
		this.comment = comment;
	}

	public boolean isLoggedAuthor() {
		return request.getAuthor().getLoggedUser() != null;
	}

	public List<CommentData> getComments() {
		return comments;
	}

	public List<FileData> getFiles() {
		return files;
	}
}
