package cz.tendersystems.helpdesk.servlet;

import java.io.*;
import java.util.*;
import java.util.concurrent.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class BackNavigationFilter implements Filter {

	public static final String NAVIGATION_KEY = "cz.tendersystems.helpdesk.navigation";
	public static final String NO_ELEMENT = "noElement";

	private static final String httpHeaderReferer = "Referer";
	private static final int depthNavigationMax = 20;

	@Override
	public void init(FilterConfig arg0) throws ServletException {
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse answer, FilterChain refer) throws IOException, ServletException {
		HttpServletRequest httpRequest = (HttpServletRequest) request;
		HttpServletResponse httpAnsware = (HttpServletResponse) answer;

		Deque<String> navigation = null;
		String removeRefer = null;
		boolean added = false;

		if (httpRequest.getMethod().equals("GET")) {
			navigation = prepareNavigation(httpRequest);

			if (allowRemovePage(navigation, httpRequest)) {
				removeRefer = navigation.peekLast();
				navigation.pollLast();
			} else if (allowAddedPage(navigation, httpRequest)) {
				navigation.addLast(httpRequest.getHeader(httpHeaderReferer));
				added = true;
			}

			String httpPozadavekString = httpRequest.getRequestURL().toString();
			if (!navigation.isEmpty() && (navigation.peekLast().contains("?") ? navigation.peekLast().split("\\?")[0].equals(httpPozadavekString) : navigation.peekLast().equals(httpPozadavekString))) {
				navigation.pollLast();
			}
		}

		try {
			refer.doFilter(request, answer);
		} catch (IOException e) {
			throw new IOException(e);
		}

		if (navigation == null) {
			return;
		}

		if (!isHtmlPageAnsware(httpRequest, httpAnsware)) {
			if (added) {
				navigation.pollLast();
			} else if (removeRefer != null) {
				navigation.addLast(removeRefer);
			}
		} else if (navigation.size() > depthNavigationMax) {
			navigation.pollFirst();
		}
	}

	@SuppressWarnings("unchecked")
	private Deque<String> prepareNavigation(HttpServletRequest httpRequest) {
		Deque<String> navigation = (Deque<String>) httpRequest.getSession(true).getAttribute(NAVIGATION_KEY);

		if (navigation == null) {
			navigation = new LinkedBlockingDeque<>();
			httpRequest.getSession().setAttribute(NAVIGATION_KEY, navigation);
		}

		return navigation;
	}

	private boolean allowAddedPage(Deque<String> navigation, HttpServletRequest httpRequest) {
		String actualPage = httpRequest.getHeader(httpHeaderReferer);
		String nextPage = httpRequest.getRequestURL().toString();
		if (actualPage == null) {
			return false;
		}

		boolean equalityOfAcutalAndNextPage = actualPage.equals(nextPage);

		return !equalityOfAcutalAndNextPage;
	}

	private boolean allowRemovePage(Deque<String> navigation, HttpServletRequest httpRequest) {
		return !navigation.isEmpty() && navigation.peekLast().equals(BackNavigationFilter.NO_ELEMENT);
	}

	private boolean isHtmlPageAnsware(HttpServletRequest httpRequest, HttpServletResponse httpAnswer) {
		boolean requiredStatuseCode = httpAnswer.getStatus() == HttpServletResponse.SC_OK;
		boolean requiredContentType = httpAnswer.getContentType() != null && httpAnswer.getContentType().contains("text/html");

		return requiredStatuseCode && requiredContentType;
	}

	@Override
	public void destroy() {
	}
}