package cz.tendersystems.helpdesk;

import javax.annotation.*;
import javax.ejb.*;
import javax.transaction.*;
import org.jboss.arquillian.container.test.api.*;
import org.jboss.arquillian.junit.*;
import org.jboss.arquillian.persistence.*;
import org.jboss.shrinkwrap.api.*;
import org.jboss.shrinkwrap.api.spec.*;
import org.junit.*;
import org.junit.runner.*;
import cz.tendersystems.helpdesk.entity.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.service.user.*;

@RunWith(Arquillian.class)
public class UserTestArquillian {

	@Deployment
	public static JavaArchive createDeploymentpackage() {
		JavaArchive jar = ShrinkWrap.create(JavaArchive.class, "helpdesk.jar").addClass(Users.class).addClass(UsersService.class).addClass(ValidationException.class).addAsManifestResource("test-persistence.xml", "persistence.xml");

		System.out.println(jar.toString(true));

		return jar;
	}

	@EJB
	UsersService userService;

	@Resource
	UserTransaction userTransaction;

	@Test
	@UsingDataSet("datasets/users.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldFindUser() throws ValidationException {
		String nick = "test";
		Users user = userService.findUsers(nick);

		Assert.assertEquals(user.getNick(), nick);
		Assert.assertEquals(user.getNick(), nick);
		Assert.assertNotNull(user.getPassword());
		Assert.assertNotNull(user.getFirstName());
		Assert.assertNotNull(user.getLastName());
	}

	@Test(expected = ValidationException.class)
	@UsingDataSet("datasets/users.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void shouldFindUserValidationException() throws Exception {
		userService.findUsers("admin");
	}

	@Test
	@ShouldMatchDataSet(value = "datasets/expected-users.xml")
	public void shouldCreateUser() throws Exception {
		Users users = new Users();
		users.setNick("test");
		users.setFirstName("test");
		users.setLastName("test");
		users.setPassword("test");

		String nick = userService.createUsers(users);

		Users user = userService.findUsers(nick);
		Assert.assertEquals(user.getNick(), nick);
	}

	@Test(expected = ValidationException.class)
	@UsingDataSet("datasets/users.xml")
	@ShouldMatchDataSet(value = "datasets/expected-users.xml")
	@Cleanup(phase = TestExecutionPhase.BEFORE)
	public void test() throws Exception {
		Users user = new Users();
		user.setNick("test");
		userService.createUsers(user);
	}

}
