package cz.tendersystems.helpdesk.service.user;

import javax.ejb.*;
import javax.persistence.*;
import cz.tendersystems.helpdesk.entity.*;
import cz.tendersystems.helpdesk.exception.*;

@Stateless
@LocalBean
public class UsersService {

	@PersistenceContext
	EntityManager em;

	public Users findUsers(String nick) throws ValidationException {
		Users users = em.find(Users.class, nick);

		if (users == null) {
			throw new ValidationException("users not exists");
		}

		return users;
	}

	public Users findUsers(String nick, EntityManager mock) {
		Users users = mock.find(Users.class, nick);

		return users;
	}

	public String createUsers(Users users) throws ValidationException {
		if (!isNickAvailable(users.getNick())) {
			throw new ValidationException("Nick already exits", users.getNick());
		}

		em.persist(users);
		em.flush();

		return users.getNick();
	}

	private boolean isNickAvailable(String nick) {
		return (boolean) em.createQuery("SELECT CASE WHEN COUNT(u) > 0 THEN false ELSE true END FROM Users u WHERE u.nick = :nick").setParameter("nick", nick).getSingleResult();
	}

	public void setEm(EntityManager em) {
		this.em = em;
	}
}
