package cz.tendersystems.helpdesk;

import java.io.*;
import javax.persistence.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.service.user.*;

public class UserTestJUnit {

	private EntityManager em;
	private UsersService userService;

	@Before
	public void setup() throws IOException {

		userService = new UsersService();
		em = Persistence.createEntityManagerFactory("helpdesk-test").createEntityManager();
		userService.setEm(em);
		em.getTransaction().begin();
	}

	@After
	public void tearDown() {
		em.getTransaction().rollback();
		em.close();
	}

	@Test
	public void shouldFindUser() throws ValidationException {
		String nick = "admin";

		Users user = userService.findUsers(nick);

		Assert.assertNotNull(user);
		Assert.assertEquals(user.getNick(), nick);
		Assert.assertNotNull(user.getPassword());
		Assert.assertNotNull(user.getFirstName());
		Assert.assertNotNull(user.getLastName());
	}

	@Test(expected = ValidationException.class)
	public void shouldFindUserValidationException() throws Exception {
		userService.findUsers("test");
	}

	@Test
	public void shouldCreateUser() throws ValidationException {
		String nick = "test";
		String password = "A8F5F167F44F4964E6C998DEE827110C";
		String firstName = "Zbyněk";
		String lastName = "Richter";

		Users user = new Users();
		user.setNick(nick);
		user.setPassword(password);
		user.setFirstName(firstName);
		user.setLastName(lastName);

		Users findUser = em.find(Users.class, userService.createUsers(user));

		Assert.assertEquals(findUser.getNick(), nick);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getFirstName(), firstName);
		Assert.assertEquals(findUser.getLastName(), lastName);
	}

	@Test(expected = ValidationException.class)
	public void shouldCreateUserValidationException() throws Exception {
		Users user = new Users();
		user.setNick("admin");
		userService.createUsers(user);
	}
}
