package cz.tendersystems.helpdesk;

import javax.persistence.*;
import org.junit.*;
import org.junit.runner.*;
import org.mockito.*;
import org.mockito.runners.*;
import cz.tendersystems.helpdesk.entity.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.service.user.*;

@RunWith(MockitoJUnitRunner.class)
public class UserTestMockito {

	@Mock
	private EntityManager mockedEm;

	@Mock
	private UsersService userService;

	@Test
	public void shouldFindUser() throws ValidationException {
		userService = new UsersService();
		userService.setEm(mockedEm);

		Users users = new Users();
		users.setNick("test");
		users.setFirstName("test");
		users.setLastName("test");
		users.setPassword("test");

		// Definovani chovani mock objektu
		Mockito.when(mockedEm.find(Users.class, "test")).thenReturn(users);

		Users user = userService.findUsers("test");
		Assert.assertNotNull(user);
		Assert.assertEquals(user.getNick(), "test");
		Assert.assertNotNull(user.getPassword());
		Assert.assertNotNull(user.getFirstName());
		Assert.assertNotNull(user.getLastName());
	}

	@Test(expected = ValidationException.class)
	public void shouldFindUserValidationException() throws Exception {
		userService = new UsersService();
		userService.setEm(mockedEm);

		Mockito.when(mockedEm.find(Users.class, "test")).thenReturn(null);

		userService.findUsers("test");
	}

	@Test
	public void shouldCreateUser() throws ValidationException {
		String nick = "test";
		String password = "A8F5F167F44F4964E6C998DEE827110C";
		String firstName = "Zbyněk";
		String lastName = "Richter";

		Users user = new Users();
		user.setNick(nick);
		user.setPassword(password);
		user.setFirstName(firstName);
		user.setLastName(lastName);

		Mockito.when(userService.createUsers(user)).thenReturn(nick);
		Mockito.when(mockedEm.find(Users.class, nick)).thenReturn(user);

		Users findUser = mockedEm.find(Users.class, userService.createUsers(user));

		Assert.assertEquals(findUser.getNick(), nick);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getFirstName(), firstName);
		Assert.assertEquals(findUser.getLastName(), lastName);
	}

	@SuppressWarnings("unchecked")
	@Test(expected = ValidationException.class)
	public void shouldCreateUserValidationException() throws Exception {
		String nick = "test";
		String password = "A8F5F167F44F4964E6C998DEE827110C";
		String firstName = "Zbyněk";
		String lastName = "Richter";

		Users user = new Users();
		user.setNick(nick);
		user.setPassword(password);
		user.setFirstName(firstName);
		user.setLastName(lastName);

		Mockito.when(userService.createUsers(user)).thenThrow(ValidationException.class);

		userService.createUsers(user);
	}
}
