package cz.tendersystems.helpdesk;

import javax.persistence.*;
import org.junit.*;
import cz.tendersystems.helpdesk.entity.*;
import cz.tendersystems.helpdesk.exception.*;
import cz.tendersystems.helpdesk.service.user.*;
import de.akquinet.jbosscc.needle.annotation.*;
import de.akquinet.jbosscc.needle.junit.*;

public class UserTestNeedle {

	public static DatabaseRule databaseRule = new DatabaseRule("helpdesk-needle");

	private final EntityManager em = databaseRule.getEntityManager();

	@Rule
	public NeedleRule needleRule = new NeedleRule(databaseRule);

	@ObjectUnderTest
	UsersService userService;

	@Before
	public void setup() {
		em.getTransaction().begin();
	}

	@After
	public void tearDown() {
		em.getTransaction().commit();
	}

	@Test
	public void shouldFindUser() throws ValidationException {
		Users testUser = new Users();
		testUser.setNick("admin");
		testUser.setFirstName("admin");
		testUser.setLastName("admin");
		testUser.setPassword("test");

		em.persist(testUser);

		String nick = "admin";

		Users user = userService.findUsers(nick);

		Assert.assertNotNull(user);
		Assert.assertEquals(user.getNick(), nick);
		Assert.assertNotNull(user.getPassword());
		Assert.assertNotNull(user.getFirstName());
		Assert.assertNotNull(user.getLastName());
	}

	@Test(expected = ValidationException.class)
	public void shouldFindUserValidationException() throws Exception {
		userService.findUsers("test");
	}

	@Test
	public void shouldCreateUser() throws ValidationException {
		String nick = "test";
		String password = "A8F5F167F44F4964E6C998DEE827110C";
		String firstName = "Zbyněk";
		String lastName = "Richter";

		Users user = new Users();
		user.setNick(nick);
		user.setPassword(password);
		user.setFirstName(firstName);
		user.setLastName(lastName);

		Users findUser = em.find(Users.class, userService.createUsers(user));

		Assert.assertEquals(findUser.getNick(), nick);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getPassword(), password);
		Assert.assertEquals(findUser.getFirstName(), firstName);
		Assert.assertEquals(findUser.getLastName(), lastName);
	}

	@Test(expected = ValidationException.class)
	public void shouldCreateUserValidationException() throws Exception {
		Users user = new Users();
		user.setNick("admin");
		userService.createUsers(user);
	}
}